{$INCLUDE AutoStatLibSettings.pas}
{$INCLUDE AutoStatLibURL.pas}

var
  akce: string;

{nahrazeni procentovych promennych skutecnejma}
function ReplaceVariable(variable: string): string;
begin
  Result := ReplaceString(variable,'%DATADIR%',GEOGET_DATADIR);
  Result := ReplaceString(variable,'%STATISTICS%',statistics);
  Result := ReplaceString(variable,'%TEMPLATE%',template);
  Result := ReplaceString(variable,'%CRLF%',CRLF);
end;

{vyhazuje okno s chybou s moznosti sestrelit autostat}
procedure ShowErrorMessage(txt: string);
begin
  if MessBox(txt + CRLF + CRLF + _('Continue?'),_('AutoStat'),4) = 7 then GeoAbort;
end;

{vyhodi okno s popisem chyby geojarryho}
procedure ShowGeojarryError(id: integer);
begin
  case id of
    0: BrowseURL('http://www.geocaching.com/profile/?u='+EncodeURLelement(AnsiToUTF(gsUsername))+'#ctl00_ContentBody_ProfilePanel1_lblProfile');
    1: ShowErrorMessage(_('GeoJarry error: Invalid CountryID!'));
    2: ShowErrorMessage(_('GeoJarry error: StateID needed!'));
    3: ShowErrorMessage(_('GeoJarry error: File not found!'));
    4: ShowErrorMessage(_('GeoJarry error: Downloaded invalid page'));
    5: ShowErrorMessage(_('GeoJarry error: The requested feature is only available for Groundspeak Premium Members!'));
    6: ShowErrorMessage(_('GeoJarry error: There is no PQ of the specified name available for download!'));
    252: ShowErrorMessage(ReplaceVariable(_('GeoJarry error: You have been banned from Groundspeak servers as you have been downloading too much and too quickly.%CRLF%You should stop downloading immediately for at least a few hours!')));
    253: ShowErrorMessage(_('GeoJarry error: Groundspeak (Geocaching.com) login error. Go to GeoJarry settings and check your options!'));
    254: ShowErrorMessage(_('GeoJarry error: HTTP communication error!'));
    255: ShowErrorMessage(_('GeoJarry error: Canceled by user'));
	  else ShowErrorMessage(_('Unknown GeoJarry error! Could not determine the result code!'));
  end;
end;

{ziskani verze z metasouboru}
function GetVersion: string;
begin
  if FileExists(GEOGET_SCRIPTDIR+'\Autostat\AutoStat.meta.ini') then
  begin
    ini := TIniFile.Create(GEOGET_SCRIPTDIR+'\Autostat\AutoStat.meta.ini');
    try
	    Result := ini.ReadString('info','version',_('unknown'));
    finally
      ini.Free;
    end;
  end
  else ShowErrorMessage(_('Error: File "AutoStat.meta.ini" was not found!'));
end;

{dialog pro ziskani domacich souradnic}
procedure SetHomeCoordinates(sender: TObject);
var
  homeLatInt, homeLonInt: extended;
begin
  homeLatInt := StrToFl(FormAdvancedSettings_EditHomeLat.Text);
  homeLonInt := StrToFl(FormAdvancedSettings_EditHomeLon.Text);
  if CoordEdit(_('Home coordinates'),_('Enter your home coordinates. Tip: Try geocoding tab!'),homeLatInt,homeLonInt) then
  begin
    if (homeLatInt = 0) or (homeLonInt = 0) then ShowMessage(_('Error: Invalid coordinates!'));
    FormAdvancedSettings_EditHomeLat.Text := FlToStr(homeLatInt);
    FormAdvancedSettings_EditHomeLon.Text := FlToStr(homeLonInt);
  end;
end;

{otevre instalacni adresar AutoStatu}
procedure OpenAutostatFolder(sender: TObject);
begin
  BrowseURL(GEOGET_SCRIPTDIR+'\AutoStat');
end;

{kontroluje zda je zadane UID cislo, jen tak pro sichr}
procedure IsValidUid(sender: TObject);
begin
  if not RegexFind('^[0-9]+$',TEdit(sender).Text) then
  begin
    if MessBox(ReplaceVariable(_('Error: UID must be number!%CRLF%Look at forum what is it?')),_('AutoStat error!'),1) = 1 then OpenUrlForumUid(nil);
    TEdit(sender).Text := '';
  end;
end;

{kontroluje zda jsou zadane souradnice ve spravnem formatu (00.000000)}
procedure IsValidCoord(sender: TObject);
begin
  if not RegexFind('^[0]?[0-9]{2}\.[0-9]{0,20}$',TEdit(sender).Text) then
  begin
    {propusti jen 12.34567 nebo 0123.456789}
    ShowMessage(ReplaceVariable(_('Error: Invalid format of coordinates. Coordinates must be%CRLF%in decimal format! For example: 50.568708 a 15.313619')));
    TEdit(sender).Text := '';
  end;
end;

{vytvoreni gjini.txt}
procedure MakeGjIni;
var
  s :string;
begin
  try
    s := '[geojarry]' + CRLF;
    s := s + 'username='+gsUsername + CRLF;
    s := s + 'password='+gsPassword + CRLF;
    s := s + 'proxyip='+GEOGET_PROXYHOST + CRLF;
    s := s + 'proxyport='+GEOGET_PROXYPORT + CRLF;
    s := s + 'proxyuser='+GEOGET_PROXYUSER + CRLF;
    s := s + 'proxypass='+GEOGET_PROXYPASS + CRLF;
    s := s + 'gpx=1' + CRLF;
    s := s + 'delay=2000' + CRLF;
    s := s + 'sessionfile='+GEOGET_SCRIPTDIR+'\geojarry\last\'+gsUsername+'.session.txt' + CRLF;
    s := s + 'datadir='+GEOGET_SCRIPTDIR+'\Autostat\last\gjtemp\' + CRLF;
    ForceDirectories(GEOGET_SCRIPTDIR+'\Autostat\last\gjtemp\');
    DeleteFiles(GEOGET_SCRIPTDIR+'\Autostat\last\gjtemp\*.*');

    if debugMode then ShowErrorMessage('gjini.txt' + CRLF + CRLF + s);
  finally
    StringToFile(s,GEOGET_SCRIPTDIR+'\Autostat\last\gjini.txt');
  end;
end;

procedure CallScript(title, script, path: string);
begin
  if debugMode then ShowErrorMessage(title+' "'+script+'"');
  if FileExists(GEOGET_SCRIPTDIR+path) then GeoCallGGP(GEOGET_SCRIPTDIR+path)
  else ShowErrorMessage(_('Error: Script "')+script+_('" not found!'));
end;

{PLUGINSTOP - nahrani na FTP}
procedure UploadOnFtp(sender: TObject);
begin
  GeoBusyKind(_('Uploading on FTP...'));

  if not (GEOGET_SCRIPTNAME = 'AutoStatSilent.ggp.pas') then SaveSettings;
  ReadSettings;

  {zkontrolujeme velikost souboru STATISTICS}
  if GetFileSize(GEOGET_SCRIPTDIR+'\AutoStat\last\'+statistics) <= StrToInt(minStatsSize) then
    ShowErrorMessage(ReplaceVariable(_('Warning: Size of file "%STATISTICS%"%CRLF%is less than 1kB. Maybe some error caused that file is blank.')));

  {posleme na FTP}
  if debugMode then ShowMessage(_('FTP upload') + CRLF + CRLF +
                                _('URL: ') + ftpUrl + CRLF +
                                _('File on FTP: ') + ftpDirectory+statistics + CRLF +
                                _('Local file: ') + GEOGET_SCRIPTDIR+'\AutoStat\last\'+statistics + CRLF +
                                _('Username: ') + ftpName + CRLF +
                                _('Password: ') + ftpPassword);

  if not FtpPutFile(ftpUrl,'21',ftpDirectory+statistics,GEOGET_SCRIPTDIR+'\AutoStat\last\'+statistics,ftpName,ftpPassword) then
    ShowErrorMessage(_('Error: File was not uploaded successfully!'))
  else if debugMode then ShowErrorMessage(_('File was uploaded successfully!'));
end;

procedure UploadOnProfile(sender: TObject);
begin
  GeoBusyKind(_('Uploading on profile...'));

  {zkontrolujeme velikost souboru STATISTICS}
  if GetFileSize(GEOGET_SCRIPTDIR+'\AutoStat\last\'+statistics) <= StrToInt(minStatsSize) then
    ShowErrorMessage(ReplaceVariable(_('Warning: Size of file "%STATISTICS%"%CRLF%is less than 1kB. Maybe some error caused that file is blank.')));

  if FileExists(GEOGET_SCRIPTDIR+'\AutoStat\last\'+statistics) then
  begin
    {prikaz pro zavolani geojarryho}
    akce := '"'+GEOGET_SCRIPTDIR+'\geojarry\geojarryw.exe" "'+GEOGET_SCRIPTDIR+'\Autostat\last\gjini.txt" profile "'+GEOGET_SCRIPTDIR+'\AutoStat\last\'+statistics+'"';
    if debugMode then ShowErrorMessage(_('Upload stats by GeoJarry:') + CRLF + CRLF + akce);
    {upload na profil pres geojarry}
    ShowGeojarryError(RunExec(akce));
  end
  else
    ShowErrorMessage(ReplaceVariable(_('Error: File "%STATISTICS%" not found!')));
end;

{jen rozhodovaci logika pro tlacitko pro okamzity upload}
procedure UploadNow(sender: TObject);
begin
  if FormAdvancedSettings_RbProfile.Checked then
    UploadOnProfile(nil)
  else
    UploadOnFTP(nil);
end;
