//GeoGet 2
var
  wpstream: TFileStream;

function ExportExtension: string;
begin
  result := 'GPX';
end;

function ExportDescription: string;
begin
  result := 'Original PocketQuery GPX';
end;

function ExportHint: string;
begin
  result := 'Basic exchange format for other apliacations. Waypoints are stored in a separate file.';
  if copy(GetCurrentLanguage, 1, 2) = 'cs' then
    result := 'Zkladn vmnn formt pro ciz apliakce. Waypointy jsou ukldny do samostatnho souboru.';
end;

function ExportHeader: string;
var
  r: string;
begin
  Result := '<?xml version="1.0" encoding="utf-8"?>' + CRLF;
  Result := Result + '<gpx xmlns:xsd="http://www.w3.org/2001/XMLSchema"'
    + ' xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" version="1.0"'
    + ' creator="Groundspeak Pocket Query"'
    + ' xsi:schemaLocation="http://www.topografix.com/GPX/1/0 http://www.topografix.com/GPX/1/0/gpx.xsd'
    + ' http://www.groundspeak.com/cache/1/0/1 http://www.groundspeak.com/cache/1/0/1/cache.xsd"'
    + ' xmlns="http://www.topografix.com/GPX/1/0">' + CRLF;
  Result := Result + ' <name>Geocaches</name>' + CRLF;
  Result := Result + ' <desc>Geocache file</desc>' + CRLF;
  Result := Result + ' <author>' + Geoget_Version + '</author>' + CRLF;
  Result := Result + ' <time>' + formatdatetime('yyyy"-"mm"-"dd"T"hh":"nn":"ss"."zzz', now) + '</time>' + CRLF;
  Result := Result + ' <keywords>cache, geocache, groundspeak</keywords>' + CRLF + CRLF;

  R := '<?xml version="1.0" encoding="utf-8"?>' + CRLF;
  R := R + '<gpx xmlns:xsd="http://www.w3.org/2001/XMLSchema"'
    + ' xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" version="1.0"'
    + ' creator="Groundspeak Pocket Query"'
    + ' xsi:schemaLocation="http://www.topografix.com/GPX/1/0 http://www.topografix.com/GPX/1/0/gpx.xsd"'
    + ' xmlns="http://www.topografix.com/GPX/1/0">' + CRLF;
  R := R + ' <name>Waypoints for Cache Listings</name>' + CRLF;
  R := R + ' <desc>This is a list of supporting waypoints for caches</desc>' + CRLF;
  R := R + ' <author>' + Geoget_Version + '</author>' + CRLF;
  R := R + ' <time>' + formatdatetime('yyyy"-"mm"-"dd"T"hh":"nn":"ss"."zzz', now) + '</time>' + CRLF;
  R := R + ' <keywords>cache, geocache, waypoints</keywords>' + CRLF + CRLF;
  wpstream := TFileStream.create(geoget_datadir + '\wpexport.tmp', fmCreate or fmShareDenyWrite);
  wpstream.write(R, length(R));
end;

function ExportFooter: string;
begin
  result := '</gpx>' + CRLF;
  
  wpstream.write(result, length(result));
  wpstream.free;  
end;

function ExportAttributes: string;
var
  l: TStringList;
  n: integer;
  aicon: string;
  aid: string;
  apq: string;
  ainc: string;
begin
  Result := '   <groundspeak:attributes>' + CRLF;
  l := TstringList.Create;
  try
    l.text := GC.TagValues('attribute');
    for n := 0 to l.count - 1 do
    begin
      aicon := trim(l[n]);
      aid := IconToAttributeID(aicon);
      if aid = '' then
        continue;
      apq := IconToAttributePQ(aicon);
      if apq <> '' then
        if apq[1] = '-' then
          delete(apq, 1, 1);
      ainc := '1';
      if aid <> '' then
        if aid[1] = '-' then
        begin
          delete(aid, 1, 1);
          ainc := '0';
        end;
      Result := Result + '     <groundspeak:attribute id="' + aid + '" inc="' + ainc + '">'
        + apq + '</groundspeak:attribute>' + CRLF;
    end;
  finally
    l.free;
  end;
  Result := Result + '   </groundspeak:attributes>' + CRLF;
end;

function ExportPoint: string;
var
  s: string;
  n: integer;
  r: string;
begin
  Result := '';
  //Export for Geocaches
  if GC.IsListed then
  begin
    Result := Result + ' <wpt lat="' + GC.Lat + '" lon="' + GC.Lon + '">' + CRLF;
    Result := Result + '  <time>'
      + formatdatetime('yyyy"-"mm"-"dd"T"hh":"nn":"ss"."zzz', GC.Hidden) + '</time>' + CRLF;
    Result := Result + '  <name>' + GC.ID +'</name>' + CRLF;
    Result := Result + '  <desc>' + cdata(GC.Name + ' by ' + GC.Author
          + ' (' + GC.Difficulty + '/' + GC.Terrain + ')') +'</desc>' + CRLF;
    Result := Result + '  <url>' + cdata(GC.URL) +'</url>' + CRLF;
    Result := Result + '  <urlname>' + cdata(GC.Name) + '</urlname>' + CRLF;
    if GC.IsFound then
      Result := Result + '  <sym>Geocache Found</sym>' + CRLF
    else
      Result := Result + '  <sym>Geocache</sym>' + CRLF;
    Result := Result + '  <type>Geocache|' + GC.CacheType + '</type>' + CRLF;
    if GC.IsDisabled then
      s := 'available="False"'
    else
      s := 'available="True"';
    if GC.IsArchived then
      s := s + ' archived="True"'
    else
      s := s + ' archived="False"';
    s := 'id="' + GC.CacheID +'" ' + s;
    Result := Result + '  <groundspeak:cache ' + s + ' xmlns:groundspeak="http://www.groundspeak.com/cache/1/0/1">' + CRLF;
    Result := Result + '   <groundspeak:name>' + cdata(GC.Name) + '</groundspeak:name>' + CRLF;
    Result := Result + '   <groundspeak:placed_by>' + cdata(GC.Author) +'</groundspeak:placed_by>' + CRLF;
    if GC.OwnerID <> '' then
      Result := Result + '   <groundspeak:owner id="' + GC.OwnerID + '">' + cdata(GC.Author) +'</groundspeak:owner>' + CRLF;
    Result := Result + '   <groundspeak:type>' + GC.CacheType +'</groundspeak:type>' + CRLF;
    Result := Result + '   <groundspeak:container>' + GC.Size +'</groundspeak:container>' + CRLF;
    Result := Result + ExportAttributes;
    Result := Result + '   <groundspeak:difficulty>' + GC.Difficulty +'</groundspeak:difficulty>' + CRLF;
    Result := Result + '   <groundspeak:terrain>' + GC.Terrain +'</groundspeak:terrain>' + CRLF;
    Result := Result + '   <groundspeak:country>' + GC.Country +'</groundspeak:country>' + CRLF;
    Result := Result + '   <groundspeak:state>' + GC.State +'</groundspeak:state>' + CRLF;
    Result := Result + '   <groundspeak:short_description html="True">' + cdata(GC.ShortDescription) +'</groundspeak:short_description>' + CRLF;
    Result := Result + '   <groundspeak:long_description html="True">' + cdata(GC.LongDescription) +'</groundspeak:long_description>' + CRLF;
    Result := Result + '   <groundspeak:encoded_hints>' + cdata(GC.Hint) +'</groundspeak:encoded_hints>' + CRLF;
    Result := Result + '   <groundspeak:logs>' + CRLF;
    for n := 0 to GC.Logs.Count - 1 do
    begin
      Result := Result + '    <groundspeak:log id="' + GC.Logs[n].LogID + '">' + CRLF;
      Result := Result + '     <groundspeak:date>' 
        + formatdatetime('yyyy"-"mm"-"dd"T"hh":"nn":"ss"."zzz', GC.Logs[n].Date) + '</groundspeak:date>' + CRLF;
      Result := Result + '     <groundspeak:type>' + GC.Logs[n].LogType + '</groundspeak:type>' + CRLF;
      s := '';
      if GC.Logs[n].FinderID <> '' then
        s := ' id="' + GC.Logs[n].FinderID + '"'; 
      Result := Result + '     <groundspeak:finder' + s + '>' + cdata(GC.Logs[n].Finder) + '</groundspeak:finder>' + CRLF;
      Result := Result + '     <groundspeak:text encoded="False">' + cdata(GC.Logs[n].Text) + '</groundspeak:text>' + CRLF;
      Result := Result + '    </groundspeak:log>' + CRLF;
    end;
    Result := Result + '   </groundspeak:logs>' + CRLF;
    Result := Result + '  </groundspeak:cache>' + CRLF;
    Result := Result + ' </wpt>' + CRLF + CRLF;
  end;
  //Export for Waypoints
  r := '';
  for n := 0 to GC.Waypoints.Count - 1 do
    if GC.Waypoints[n].IsListed then
    begin
      R := R + ' <wpt lat="' + GC.Waypoints[n].Lat + '" lon="' + GC.Waypoints[n].Lon + '">' + CRLF;
      R := R + '  <time>'
        + formatdatetime('yyyy"-"mm"-"dd"T"hh":"nn":"ss"."zzz', GC.Hidden) + '</time>' + CRLF;
      R := R + '  <name>' + GC.Waypoints[n].ID +'</name>' + CRLF;
      R := R + '  <cmt>' + cdata(GC.Waypoints[n].Description) +'</cmt>' + CRLF;
      R := R + '  <desc>' + cdata(GC.Waypoints[n].Name) +'</desc>' + CRLF;
      R := R + '  <url>' + cdata(GC.Waypoints[n].URL) +'</url>' + CRLF;
      R := R + '  <urlname>' + cdata(GC.Waypoints[n].Name) + '</urlname>' + CRLF;
      R := R + '  <sym>' + GC.Waypoints[n].WptType + '</sym>' + CRLF
      R := R + '  <type>Waypoint|' + GC.Waypoints[n].WptType + '</type>' + CRLF;
      R := R + ' </wpt>' + CRLF + CRLF;
    end;
  wpstream.write(r, length(r));
end;

function ExportAfter(value: string): string;
begin
  result := '';
  delete(value, length(value)-3, 4);
  value := value + '-wpts.gpx';
  deletefile(value);
  renamefile(geoget_datadir + '\wpexport.tmp', value);
end;
