{$include gj2.config.pas}
{$include groundspeak.config.pas}

const
   GEOJARRY_VERSION = '2.3.6'; //DONT FORGET TO UPDATE GEOJARRY_COMPATIBLE IN FILE gj2.lib.pas
                               //DONT FORGET TO UPDATE geojarry.meta.ini file
                               //DONT FORGET TO UPDATE README FILES
   GEOJARRY_CONFIG = 'gj2.config.pas'; //Name of geojarry config file. DONT FORGET TO CHANGE INCLUDES IF YOU CHANGE THIS VALUE
   GROUNDSPEAK_CONFIG = 'groundspeak.config.pas'; //Name of groundspeak config file. DONT FORGET TO CHANGE INCLUDES IF YOU CHANGE THIS VALUE

   YES = 'yes';
   NO = 'no';
   
   MINIMAL_DELAY = '500';
   
{Dynamic constants}
function GEOJARRY_DIR() : String; //Returns full path to geojarry main directory
begin
    Result := GEOJARRY_DIRNAME;
    If Not (copy(Result,0,1) = '\') then Result := '\' + Result;
    If Not (copy(Result,Length(Result)-1,1) = '\') then Result := Result + '\';
    Result := GEOGET_SCRIPTDIR + Result;
end;

function LAST_DIR() : String; //Directory, where last settings and downloaded webpages reside
begin Result := GEOJARRY_DIR + 'last\'; end;

function RESULT_VALID() : String;
begin Result := _('OK'); end;

function RESULT_INVALID() : String;
begin Result := _('Please, check errors and try again.'); end;

function RESULT_NOWAYPOINTS() : String;
begin Result := _('Invalid page downloaded. There might be no waypoints which suit your search options.'); end;

function DO_NOT_EDIT() : String;
begin Result := _('Use GeoJarry settings to set GeoJarry options. Do not edit this field manually.'); end;   

var //Duplicates of settings-variables. Memory-only                                                     
  gUsername, gPassword, gHomeStateName, gHomeCountryName, gDefaultTab, gDistanceUnit, gWarning, gLog, gLogMaxsize, gDelay: String;
  
  qrun     : TDateTime; //part of filename of downloaded files
  
  tagList : TStringList; //list with instructions to tag geocaches with tags and values (bookmark tagging)
  
  imported : boolean; //did last query/batch import anything?
  hideNextImport : boolean; //should the next dialog with import results be hidden?

{Returns path of geojarry config file}
function GetGjConfigFile() : String;
begin
   Result := GEOJARRY_DIR + GEOJARRY_CONFIG;
end;

{Returns path of groundspeak config file}
function GetGsConfigFile() : String;
begin
   Result := GEOGET_SCRIPTDIR + '\' + GROUNDSPEAK_CONFIG;
end;

{Write new / replace old line in configuration file}  
function SettingsWriteOption(configfile, option, value, comment : String) : String;
var
  list : TStringList;
  n : Integer;
  added : boolean;
  newline : String;
begin
  Result := _('General failure writing settings to file');
  
  added := false;
  newline := '  ' + option + ' = '''+ value +''';';
  if comment <> '' then newline := newline + ' //' + comment;
  
  list := TStringList.Create;
  try
    if (FileExists(configfile)) then
    begin
      list.LoadFromFile(configfile);
      for n := 0 to list.count-1 do
      begin
        if RegexFind(option + '(\s)*=(\s)*.*;', list[n]) then 
        begin
          list[n] := newline;
          added := true;
          break;
        end 
      end;
      if not added then list.add(newline);
    
      list.SaveToFile(configfile);
      Result := RESULT_VALID;
    end else 
      begin
        Result := _('Could not find config file: %FILE%');
        Result := ReplaceString(Result,'%FILE%',configFile);
      end;
  finally
    list.Free;
  end;
end;

{Write settings - save settings from memory to config file}  
function SettingsWrite : String;
begin
  Result := SettingsWriteOption(GetGsConfigFile(),'gsUsername',gUsername,'Groundspeak username (Geocaching.com, Waymarking.com, ...)'); if Result = RESULT_VALID then
  Result := SettingsWriteOption(GetGsConfigFile(),'gsPassword',gPassword,'Groundspeak password (Geocaching.com, Waymarking.com, ...)'); if Result = RESULT_VALID then
  Result := SettingsWriteOption(GetGjConfigFile(),'gjHomeCountryName',gHomeCountryName,DO_NOT_EDIT); if Result = RESULT_VALID then
  Result := SettingsWriteOption(GetGjConfigFile(),'gjHomeStateName',gHomeStateName,DO_NOT_EDIT); if Result = RESULT_VALID then
  Result := SettingsWriteOption(GetGjConfigFile(),'gjDefaultTab',gDefaultTab,DO_NOT_EDIT);  if Result = RESULT_VALID then
  Result := SettingsWriteOption(GetGjConfigFile(),'gjDistanceUnit',gDistanceUnit,DO_NOT_EDIT); if Result = RESULT_VALID then
  Result := SettingsWriteOption(GetGjConfigFile(),'gjWarning',gWarning,DO_NOT_EDIT); if Result = RESULT_VALID then
  Result := SettingsWriteOption(GetGjConfigFile(),'gjLog',gLog,DO_NOT_EDIT); if Result = RESULT_VALID then
  Result := SettingsWriteOption(GetGjConfigFile(),'gjLogMaxsize',gLogMaxsize,DO_NOT_EDIT); if Result = RESULT_VALID then 
  Result := SettingsWriteOption(GetGjConfigFile(),'gjDelay',gDelay,DO_NOT_EDIT); //dont forget to add another if clause when adding new option
end;

{Convert boolean from text interpretation to boolean}
function StringBoolToBool(str : String) : Boolean;
begin
   Result := (str = YES); 
end;

{Convert boolean to text interpretation of boolean}
function BoolToStringBool(str: boolean) : String;
begin
   if str 
     then Result := YES
     else Result := NO;
end;

{Show the warning in the main form?}
procedure SetWarning(warning : boolean);
begin
   if (gj2main_AllLabelWarning.Visible <> warning) then
     if warning 
       then gj2main.height := gj2main.height + 70
       else gj2main.height := gj2main.height - 70;
   
   gj2main_AllLabelWarning.Visible := warning;
   gj2main_AllLabelWarning2.Visible := warning;
   gj2main_AllLabelWarning3.Visible := warning;
   gj2main_AllLabelWarning4.Visible := warning;
end;
  
{$include gj2settings.lib.pas}

{Load settings from config file}  
procedure SettingsLoad;
var
  res,msg : String;
begin
  gUsername := gsUsername;
  gPassword := gsPassword;
  gHomeCountryName := gjHomeCountryName;
  gHomeStateName := gjHomeStateName;
  gDefaultTab := gjDefaultTab;
  gDistanceUnit := gjDistanceUnit;
  gWarning := gjWarning;
  gLog := gjLog;
  gLogMaxsize := gjLogMaxsize;
  gDelay := gjDelay;
  
  if GEOGET_OWNER = '' then ShowMessage(_('Your GeoGet software is not configured correctly.' + CRLF + 'GeoGet will not be able to recognize your found caches until you set your Geocaching.com login in GeoGet settings.' + CRLF + 'Quit GeoJarry, go to "Tools -> Settings -> Settings -> My account name" and fill in your username.')); 
  
  SetWarning(StringBoolToBool(gWarning)); 
  if (gHomeCountryName = '') then gHomeCountryName := GEOGET_DB.GetTableString('SELECT country FROM geocache GROUP BY country ORDER BY count(ID) DESC LIMIT 1;'); //We try to be smart and fill the most numerous country in the database
  if StrToInt(gDefaultTab) <> -1 then gj2main_AllPageControl.TabIndex := StrToInt(gDefaultTab);  

  if gUsername = '' then gUsername := GEOGET_OWNER; //We try to be smart and fill the same username as the GeoGet owner is
  if gUsername = '' then ShowMessage(_('You didn''t set your Groundspeak (Geocaching.com) login in GeoGet options. No waypoints will be recognized as found.'));
  if (gPassword = '') then GeoJarrysettingsShow; //If we don't have the password, we have to obtain it 
  if (gDelay = '') then gDelay := '500';
  if (StrToInt(gDelay) < StrToInt(MINIMAL_DELAY)) then 
  begin 
    gDelay := MINIMAL_DELAY; 
    msg := _('Download delay must not be set below %MINIMAL_DELAY%ms. Automaticaly reseted to %MINIMAL_DELAY%ms.');
    msg := ReplaceString(msg,'%MINIMAL_DELAY%',MINIMAL_DELAY);
    ShowMessage(msg); 
  end;
  
  res := SettingsWrite;
  if res <> RESULT_VALID then ShowMessage(res);
end; 

{Get geojarry executable}
function GetGeoJarryExe: String;
var msg : String;
begin
   Result := GEOJARRY_DIR + GEOJARRY_EXE;
   if Not FileExists(Result) then 
   begin
      msg := _('Could not find file %FILE%.' + CRLF + 'GeoJarry can not download any data.');
      msg := ReplaceString(msg,'%FILE%',Result);
      ShowMessage(msg);
      Result := RESULT_INVALID;
   end;
end;

{Remove all invalid filename chars from the string}
function StringToFileName(str : String) : String;
begin
    Result := RegexReplace('[\\/:*?\"<>|'']',str,'',false);
end;
 
{Get path of file with saved Groundspeak session}
function GetGeoJarrySession : String;
begin
   Result := LAST_DIR + StringToFileName(gUsername) + '.session.txt';
end;

{Get the directory, where we store the downloaded files}
function GetDatadir : String;
begin
   Result := LAST_DIR + 'download\';
   ForceDirectories(Result);
end;

{Delete all files in the given directory}
procedure EmptyDir(directory : String);
begin
   DeleteFiles(directory + '*.*');
end;

{Is the given directory empty?}
function DirIsEmpty(directory : string) : boolean;
var
  flist : TStringList;
begin
  flist := TStringList.Create;
  try
     FileList(directory,flist);
     Result := (flist.count = 0);
  finally
     flist.Free;
  end;
end;

{Compute new prefix for next geojarry query/batch}
procedure NewGjFilename;
begin
    qrun := now;
end;

{Get the name of the prefix for next geojarry query/batch}                          
function GetGjFilename : String;
begin
   if qrun = 0 then NewGjFilename;
   Result := 'gj.' + FormatDateTime('yyyymmddhhnnss',qrun);
end;

{Get the location of downloaded geocache code list}
function GetDownloadedGcListFile : String;
begin
  Result := GetDatadir + GetGjFilename + '.gclist.txt';
end;

{Get the location of downloaded raw html webpage}
function GetDownloadedRawFile : String;
begin
  Result := GetDatadir + GetGjFilename + '.raw.htm';
end;

{Get the part of geojarry ini part, which is the same for all commands}
function GetIniHeader : String;
begin
  Result := 
  '[geojarry]' + CRLF +
  'username=' + gUsername + CRLF +
  'password=' + gPassword + CRLF + 
  'proxyip=' + GEOGET_PROXYHOST + CRLF +
  'proxyport=' + GEOGET_PROXYPORT + CRLF +
  'proxyuser=' + GEOGET_PROXYUSER + CRLF +
  'proxypass=' + GEOGET_PROXYPASS + CRLF +
  'datadir=' + GetDatadir + CRLF +
  'sessionfile=' + GetGeoJarrySession + CRLF + 
  'filename='+ GetGjFilename + CRLF + 
  'delay=' + gDelay + CRLF + 
  'gpx=1' + CRLF;
end;

{Get the location of geojarry ini file}
function GetIniFile : String;
begin
  Result := LAST_DIR + 'inifile.txt';
end;

{Get the location of list of waypoints to download}
function GetListFile : String;
begin
  Result := LAST_DIR + 'listfile.ggt';
end;

{Translate the result code from geojarry.exe into human-readable format}
function GeoJarryResultToString(gj : Integer) : String;
var msg : String;
begin
   Case gj of
   0: Result := RESULT_VALID;
   1: Result := _('Invalid CountryID');
   2: Result := _('StateID needed');
   3: Result := _('File not found');
   4: Result := RESULT_NOWAYPOINTS;
   5: Result := _('The requested feature is only available for Groundspeak Premium Members.');
   6: Result := _('There is no PQ of the specified name available for download.');
   252: Result := _('You have been banned from Groundspeak servers as you have been downloading too much and too quickly.' + CRLF + 'You should stop downloading immediately for at least a few hours!');
   253: Result := _('Groundspeak (Geocaching.com) login error. Go to GeoJarry settings and check your options.');
   254: Result := _('HTTP communication error');
   255 : Result := _('Canceled');
   Else Result := _('Unknown GeoJarry error! Could not determine the result code!');
   End;
   if ((Result <> RESULT_VALID) and (not DirIsEmpty(GetDatadir))) then //User might want to import the partially downloaded data
   begin
      msg := _('GeoJarry executable prematurely stopped working with result:' + CRLF + '%RESULT%' + CRLF + CRLF + 'However, you might want to import the partially downloaded data, which can be found in folder' + CRLF + '%DATADIR%.' + CRLF + CRLF + 'You can do so in GeoGet menu "File -> Import -> Import Folder..."');
      msg := ReplaceString(msg,'%RESULT%',Result);
      msg := ReplaceString(msg,'%DATADIR%',GetDatadir);
      Result := msg; 
   end;
end;

{Remove all non-(GC or WM) codes from list}
function CodeListRemoveInvalid(codelist : TStringList) : TStringList;
var
  n : Integer;
begin
    Result := TStringList.Create;
    for n := 0 to codelist.count-1 do
    begin
        if (IsValidGC(codelist[n]) or IsValidWM(codelist[n])) then Result.add(codelist[n]);
    end;
end;

function GeoJarryDateTimeToString(dt : TDateTime) : String;
begin
    Result := FormatDateTime('yyyy"-"mm"-"dd"T"hh":"mm":"ss"."zzz',dt);
end;

{Log the last used command and ini to logfile}
procedure GeoJarryLog(command,ini : String);
var file,log : String;
begin
   file := LAST_DIR + 'geojarry.log';   
   if GetFileSize(file) > (StrToInt(gLogMaxsize) * 1024) then DeleteFile(file); 
   
   if FileExists(file) then log := FileToString(file);
   log := log 
   + '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::' + CRLF
   + '::::::::::::::::::::::::::: ' + GeoJarryDateTimeToString(now) + ' ::::::::::::::::::::::::::::' + CRLF
   + '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::' + CRLF
   + '::::::::::::::::::::::::::::::::::: COMMAND ::::::::::::::::::::::::::::::::::::' + CRLF
   + command + CRLF
   + '::::::::::::::::::::::::::::::::::: INIFILE ::::::::::::::::::::::::::::::::::::' + CRLF
   + ini + CRLF
   + '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::' + CRLF
   + CRLF + CRLF
   StringToFile(log,file); 
end;

{RunExec with logging to file}
function GeoJarryRunExec(Value : String) : Integer;
var ini : String;
begin
    ini := RegexSubstitute('(?i)^\s*"[^"]*?\.exe"\s+"([^"]*)"',Value,'$1');
    if FileExists(ini) then ini := FileToString(ini) else ini := _('Log error: could not find the ini file specified by the command.');
    if (StringBoolToBool(gLog)) then GeoJarryLog(Value, ini);
    Result := RunExec(value);
end;

{Download all listings for waypoints included in the list}
function DownloadWaypointListings(list : TStringList; fullListings: boolean) : String;
var
  Command : String;
begin
  Result := '';
  
  //Remove non-(GC or WM) codes
  list := CodeListRemoveInvalid(list);
  
  //Inifile
  StringToFile(GetIniHeader, GetIniFile);
  
  //Command
  Command := '"' + GetGeoJarryExe + '" "' + GetIniFile + '" caches "' + GetListFile + '"';  
  if fullListings then Command := Command + ' all';
  
  //Write list
  DeleteFile(GetListFile);
  list.SaveToFile(GetListFile);

  //Download
  Result := GeoJarryResultToString(GeoJarryRunExec(Command));
end;

procedure AddRequestToTagList(category,value : String; deleteBefore : boolean; list : TStringList);
var 
  taglisttag : String;
begin
   taglisttag := AnsiToUtf(category) + '|' + AnsiToUtf(value)+ '|' + BoolToStringBool(deleteBefore);
   taglist.addObject(taglisttag,list);
end;

{Delete all occurrences of this tag and value in the database}
procedure DeleteAllTagOccurrences(category,value : String);
var
   s : String;
begin
    //Busy
    s := _('Deleting all tag occurrences (%CATEGORY% - %VALUE%)');
    s := ReplaceString(s,'%CATEGORY%',UtfToAnsi(category));
    s := ReplaceString(s,'%VALUE%',UtfToAnsi(value));
    GeoBusyCaption(_('GeoJarry')); GeoBusyKind(s); GeoBusyValue('');
    
    //Delete
    GEOGET_DB.AddParamText(':category',category);
    GEOGET_DB.AddParamText(':value',value);
    GEOGET_DB.ExecSQL('DELETE FROM geotag WHERE ptrkat = (SELECT key FROM geotagcategory WHERE value = :category) AND ptrvalue = (SELECT key FROM geotagvalue WHERE value = :value);');
    
    //Busy
    GeoBusyCaption(''); GeoBusyKind(''); GeoBusyValue('');
end;

{Add tag of this category and value to all caches in the list}
procedure AddTagToCaches(category,value : String; list : TStrings);
var 
  s : String;
  n,max : Integer;
  geo : TGeo;
begin
    //Busy
    s := _('Adding tags to geocaches (%CATEGORY% - %VALUE%)');
    s := ReplaceString(s,'%CATEGORY%',UtfToAnsi(category));
    s := ReplaceString(s,'%VALUE%',UtfToAnsi(value));
    GeoBusyCaption(_('GeoJarry')); GeoBusyKind(s); GeoBusyValue('');
    max := list.count;

    //Add tags
    geo := TGeo.Create;
    try
      for n := 0 to max-1 do
      begin
          if not GeoBusyTest then
          begin
            GeoBusyProgress(n+1,max);
            geo.clear;
            geo.id := list[n];
            geo.TagAdd(category, value);
          end;
      end;
    finally
      geo.Free;
    end;

    //Busy
    GeoBusyCaption(''); GeoBusyKind(''); GeoBusyValue('');
end;

{Process the taglist - add tags to the desired caches}
procedure ProcessTagList;
var 
  n : Integer;
  line,category,value : String;
  delete : boolean;
  list : TStringList;
begin
    for n := 0 to tagList.Count-1 do
    begin
        if not GeoBusyTest then
        begin
          line := tagList[n];
          category := fetch(line,'|');
          value := fetch(line,'|');
          delete := StringBoolToBool(fetch(line,'|'));
          list := TStringList(tagList.Objects[n]);
          
          if delete then DeleteAllTagOccurrences(category,value);
          AddTagToCaches(category,value,list);
          list.Free;
        end;
    end;
    tagList.Clear;
end;

{Clear statistics of imported files}
procedure GeoJarryGeoImportBegin;
begin
   imported := false;
   GeoImportBegin;
end;

{Import all files in the folder}
procedure GeoJarryGeoImportDir(dir : String);
var
  files : TStringList;
begin
   GeoImportDir(dir);
   ProcessTagList;
   files :=  TStringList.Create;
   try
     FileList(dir, files)
     if (files.count > 0) then imported := true;
   finally
     files.free;
   end;
end;

{Show result of import}
procedure GeoJarryGeoImportEnd;
begin
   if ((imported) and (not hideNextImport))then GeoImportEnd; 
   imported := false;
   hideNextImport := false;
end;

{Free memory used by forms} 
procedure FormReleaseAll; 
begin
  gj2main.release;
  gj2web.release;
  gj2geocoding.release;
  gj2batch.release;
  gj2batchquery.release;
  gj2db.release;
  gj2wm.release;
  gj2plugin.release;
  gj2login.release;
  gj2bookmark.release;
end;

{Set all necesarry things in the forms. Setup events, default values}
procedure SetCommonActions;
begin
   GeoJarrysettingsSetActions;
end;

{Check if everything necessary is on place}
function CheckEnvironment : boolean;
var
   res : String;
begin
   res := GetGeoJarryExe;
   Result := (res <> RESULT_INVALID);
   if not Result then 
   begin
     ShowMessage(_('GeoJarry found one or more problems in environment and will now exit. Please, check your installation.'));
     GeoAbort;
   end;
end;

{Initialize taglist}
procedure InitTagList;
begin
    tagList := TStringList.Create;
end;

{Free taglist}
procedure FreeTagList;
var n : Integer;
begin
    for n := 0 to tagList.Count-1 do 
    begin
      TStringList(tagList.Objects[n]).Free;
    end;
    tagList.Free;
end;

{Initialize}
procedure InitCommon;
begin
   CheckEnvironment;
   SetCommonActions;
   SettingsLoad;
   InitTagList
end;

{Cleanup}
procedure CleanupCommon;
begin
   FreeTagList;
   FormReleaseAll;
end;

{$include gj2combo.lib.pas}
