var
   batchfile : String; //a variable holding information about name of currently processed batch

{Show information string about a query - determine the type and delegate}   
function QueryInfo(file : String) : String;
begin
    case GetQueryBase(file) of 
    QUERYBASE_WEB : Result := GeoJarrywebQueryInfo(file);
    QUERYBASE_WM : Result := GeoJarrywmQueryInfo(file);
    QUERYBASE_DB : Result := GeoJarrydbQueryInfo(file);
    else 
      begin
        Result := _('Could not determine query type or file not found:' + CRLF + '%FILE%');
        Result := ReplaceString(Result,'%FILE%',file);
      end 
    end;                                                        
end;

{Extract the command name from localized string}
function GeojarryBatchCommandToCommandName(command : String) : String;
begin
    Result := RegexSubstitute('\((.+)\)$',command,'$1');
end;

{Get the localized string of command long name from command id}
function GeojarryBatchCommandIdToCommandName(name : String) : String;
begin
   case name of
   'GgClearWindow':       begin Result := _('GeoGet - clear window'); End;
   'GcGetFinds':          begin Result := _('Geocaching.com - get dates of my finds'); End;
   'GsLoginChange':       begin Result := _('Groundspeak - change login'); End;
   'GsLoginReset':        begin Result := _('Groundspeak - reset login'); End;
   'GgHideImport':        begin Result := _('GeoGet - hide next import results'); End;
   'GgDbChange':          begin Result := _('GeoGet - change database'); End;
   'GgImport':            begin Result := _('GeoGet - import downloaded queries'); End;
   'PendClear':           begin Result := _('Pending list - clear'); End;
   'PendDown':            begin Result := _('Pending list - download'); End;
   'PendDownAll':         begin Result := _('Pending list - download + all logs'); End;
   'PendOutDb':           begin Result := _('Pending list - filter out database'); End;
   'PendOutFndOwn':       begin Result := _('Pending list - filter out found/owned'); End;
   'PendLoadWindow':      begin Result := _('Pending list - load to GeoGet window'); End;
   'PendSave':            begin Result := _('Pending list - save as GGT list'); End;
   'PendSub':             begin Result := _('Pending list - subtract from GeoGet window'); End;
   'GgRunGGP':            begin Result := _('GeoGet - run GeoGet GGP plugin'); End;
   'GgSetRefPoint':       begin Result := _('GeoGet - set reference point'); End;
   
   'QueryAdd':            begin Result := _('Query - add to pending list'); End;
   'QueryDown':           begin Result := _('Query - download now'); End;
   'QuerySub':            begin Result := _('Query - subtract from pending list'); End;
   Else Result := _('Could not recognize command:') + name;
   End;
end;

{Get stringlist of available batch commands}
function GeojarryBatchAvailableCommands : TStringList;
var n : Integer;
begin
    Result := TStringList.Create;
    Result.Add('GgClearWindow');
    Result.Add('GcGetFinds');
    Result.Add('GsLoginChange');
    Result.Add('GsLoginReset');
    Result.Add('GgHideImport');
    Result.Add('GgDbChange');
    Result.Add('GgImport');
    Result.Add('PendClear');
    Result.Add('PendDown');
    Result.Add('PendDownAll');
    Result.Add('PendOutDb');
    Result.Add('PendOutFndOwn');
    Result.Add('PendLoadWindow');
    Result.Add('PendSave');
    Result.Add('PendSub');
    Result.Add('GgRunGGP');
    Result.Add('GgSetRefPoint');
    
    for n := 0 to Result.count-1 do
    begin
        Result[n] := GeojarryBatchCommandIdToCommandName(Result[n]) + ' (' + Result[n] + ')';
    end; 
end;  

{Show information string about a command}
function GeojarryBatchCommandInfo(name : String) : String;
begin
   case name of
   'GgClearWindow':       begin Result := _('This command will clear list of loaded waypoints in GeoGet window.'); End;
   'GcGetFinds':          begin Result := _('This command searches Geocaching.com and downloads dates of finds of all geocaches you already found. It can not import any new geocaches, but only set date of find on geocaches already present in the database. This command will not download your logs.'); End;
   'GsLoginChange':       begin Result := _('This command will temporarily change your Groundspeak login (Geocaching.com and Waymarking.com username and password used by GeoJarry to access the webpages) to the values specified. Next GeoJarry session will use login from saved configuration file again. You can restore the default login even for the current session using "Groundspeak - reset login" command.'); End;
   'GsLoginReset':        begin Result := _('This command will reload your Groundspeak login (Geocaching.com and Waymarking.com username and password used by GeoJarry to access the webpages) from configuration file.'); End;
   'GgHideImport':        begin Result := _('This command will force GeoJarry not to show next import results dialog. It might be useful if you plan to save this batch as plugin and run it as a part of another batch. Only use it if you know what you are doing.'); End;
   'GgDbChange':          begin Result := _('This command will open and activate GeoGet database of your choice. All queries imported after this change will be imported to the new database.'); End;
   'GgImport':            begin Result := _('This command imports queries previously downloaded in the current batch. Use it before running database queries searching for new listings or archived waypoints or before running GGP plugins adding or editing content of your database. You don''t have to add this command to the end of the batch, GeoJarry will import your queries automatically.'); End; 
   'PendClear':           begin Result := _('This command will remove all waypoints from the current pending list.'); End;
   'PendDown':            begin Result := _('This command downloads previously processed queries added to pending list. After the download, pending list will be cleared. You don''t have to add this command to the end of the batch, GeoJarry will download your queries automatically.') ;End;
   'PendDownAll':         begin Result := _('This command downloads previously processed queries added to pending list. After the download, pending list will be cleared. Unlike the "Pending list - download" command, this command downloads listings with all logs. Use it only if you know what you are doing.') ;End;
   'PendOutDb':           begin Result := _('Filter out waypoints already present in the database from pending list. This option is great for discovering newly published geocaches or waymarks. Create a query searching for new geocaches or waymarks in your area, add it to the pending list and then use this command to subtract waypoints already present in the database. GeoJarry will download listings of new geocaches or waymarks.'); End;
   'PendOutFndOwn':       begin Result := _('Filter out waypoints marked in database as found or owned from the pending list. This option is great to download and mark new found geocaches from the website. Create a query searching for your last found caches, add it to the pending list and then subtract finds using this command.'); End;
   'PendLoadWindow':      begin Result := _('This command will load waypoints currently contained in the pending list into GeoGet window. Only waypoints already present in the database will be loaded.'); End;
   'PendSave':            begin Result := _('Save the contents of pending list (waypoints from previously processed queries added to pending list) as GeoGet GGT list.'); End;
   'PendSub':             begin Result := _('This command will subtract waypoints currently contained in the pending list from waypoints currently loaded in GeoGet window.'); End;
   'GgRunGGP':            begin Result := _('Run any GeoGet GGP plugin you want. For example a third party plugin to load waypoints into GeoGet window, or a plugin which updates elevation and evaluation of waypoints. If you wan''t to access newly downloaded waypoints from GGP plugins, you must import the downloaded queries first!'); End;
   'GgSetRefPoint':       begin Result := _('Set current reference point to the one you choose. This option is helpful if you use relative GeoGet filters (searching by distance from current reference point).'); End;

   'QueryAdd':            begin Result := _('The query will be added to pending list.'); End;
   'QueryDown':           begin Result := _('The query will be downloaded instantly.'); End;
   'QuerySub':            begin Result := _('The query will be subtracted from pending list.'); End;
   Else Result := _('Could not recognize command:') + name;
   End;
end; 

procedure BatchListBoxIncludedOnClick(Sender : TObject);
var cmd : String;
begin
    if (gj2batch_BatchListBoxIncluded.ItemIndex <> -1) then 
    begin
      gj2batch_BatchButtonRemove.Enabled := true;
      
      if (pos('|',gj2batch_BatchListBoxIncluded.Items[gj2batch_BatchListBoxIncluded.ItemIndex]) > 0) 
        then cmd := copy(gj2batch_BatchListBoxIncluded.Items[gj2batch_BatchListBoxIncluded.ItemIndex],0,pos('|',gj2batch_BatchListBoxIncluded.Items[gj2batch_BatchListBoxIncluded.ItemIndex])-1)
        else cmd := gj2batch_BatchListBoxIncluded.Items[gj2batch_BatchListBoxIncluded.ItemIndex];
      gj2batch_BatchListBoxIncluded.Hint := GeojarryBatchCommandIdToCommandName(cmd) + ' (' + ReplaceString(gj2batch_BatchListBoxIncluded.Items[gj2batch_BatchListBoxIncluded.ItemIndex],'|',' - ') + ')';
      gj2batch_BatchLabelAbout.Caption := GeojarryBatchCommandInfo(cmd); 
        
      gj2batch_BatchButtonUp.Enabled := (gj2batch_BatchListBoxIncluded.ItemIndex <> 0);
      gj2batch_BatchButtonDown.Enabled := (gj2batch_BatchListBoxIncluded.ItemIndex <> gj2batch_BatchListBoxIncluded.Items.Count-1);

      gj2batch_BatchListBoxQueries.ItemIndex := -1;
      gj2batch_BatchListBoxCommands.ItemIndex := -1;
      gj2batch_BatchButtonAdd.Enabled := false;  
    end
    else 
    begin
      gj2batch_BatchButtonRemove.Enabled := false;
      gj2batch_BatchButtonUp.Enabled := false;
      gj2batch_BatchButtonDown.Enabled := false;
      gj2batch_BatchListBoxIncluded.Hint := _('List of commands included in this batch');
      gj2batch_BatchLabelAbout.Caption := '';
    end;
end;

procedure BatchListBoxQueriesOnClick(Sender : TObject);
begin
    if (gj2batch_BatchListBoxQueries.ItemIndex <> -1) then 
      begin 
        gj2batch_BatchListBoxIncluded.ItemIndex := -1;
        BatchListBoxIncludedOnClick(Sender);
        gj2batch_BatchListBoxCommands.ItemIndex := -1; 

        gj2batch_BatchButtonAdd.Enabled := true;
        gj2batch_BatchListBoxQueries.Hint := gj2batch_BatchListBoxQueries.Items[gj2batch_BatchListBoxQueries.ItemIndex]; 
        gj2batch_BatchLabelAbout.Caption := QueryInfo(GetQueryFile(gj2batch_BatchListBoxQueries.Items[gj2batch_BatchListBoxQueries.ItemIndex]));
      end
    else 
      begin
        gj2batch_BatchButtonAdd.Enabled := false;
        gj2batch_BatchButtonAdd.Hint := _('List of available queries');
        gj2batch_BatchLabelAbout.Caption := '';
      end;
end;

procedure BatchListBoxCommandsOnClick(Sender : TObject);
begin
    if (gj2batch_BatchListBoxCommands.ItemIndex <> -1) then
      begin
        gj2batch_BatchListBoxIncluded.ItemIndex := -1;
        BatchListBoxIncludedOnClick(Sender);
        gj2batch_BatchListBoxQueries.ItemIndex := -1;  


        gj2batch_BatchButtonAdd.Enabled := true;
        gj2batch_BatchListBoxCommands.Hint := gj2batch_BatchListBoxCommands.Items[gj2batch_BatchListBoxCommands.ItemIndex];
        gj2batch_BatchLabelAbout.Caption := GeojarryBatchCommandInfo(GeojarryBatchCommandToCommandName(gj2batch_BatchListBoxCommands.Items[gj2batch_BatchListBoxCommands.ItemIndex]));
      end
    else
      begin
        gj2batch_BatchButtonAdd.Enabled := false;
        gj2batch_BatchListBoxCommands.Hint := _('List of available commands');
        gj2batch_BatchLabelAbout.Caption := '';
      end;
end;

procedure BatchPageControlOnChange(Sender : TObject);
begin
    case gj2batch_BatchPageControl.TabIndex of
    0 : BatchListBoxQueriesOnClick(Sender);
    1 : BatchListBoxCommandsOnClick(Sender);
    Else ShowMessage(_('Error determining active page!'));
    end;
end;

procedure BatchqueryRGroupOptionOnClick(Sender: TObject);
begin
    case gj2batchquery_RGroupQueryOption.ItemIndex of
   -1 : gj2batchquery_LabelOptionDescription.Caption := _('Select query option');
    0 : gj2batchquery_LabelOptionDescription.Caption := _('GeoJarry will download this query instantly. You will not be able to filter or process the results in any way. The downloaded waypoint listings will be imported at the and of the batch processing. It is recommended to use "Add to pending list", because it ensures, that listing of one waypoint will not be downloaded more than once during the batch processing.');
    1 : gj2batchquery_LabelOptionDescription.Caption := _('Add waypoint listings from this query to the pending list. This is a list of waypoint codes, which can be filtered, saved or loaded into GeoGet window. Listings of waypoints in pending list will be downloaded and imported at the and of the batch processing. Check available commands to see what can be done with the pending list.');
    2 : gj2batchquery_LabelOptionDescription.Caption := _('Subtract (remove) waypoint listings from this query from the pending list. Listings of waypoints from this query will therefore not be downloaded. Pending list is a list of waypoint codes, which can be filtered, saved or loaded into GeoGet window. Listings of waypoints in pending list will be downloaded and imported at the and of the batch processing. Check available commands to see what can be done with the pending list.');
    Else ShowMessage(_('Error determining query option'));
    end;
end;

function BatchqueryGetQueryDownloadOption(name : String) : String;
begin
    Result := RESULT_INVALID;

    gj2batchquery.Caption := _('GeoJarry %VERSION% - query options');
    gj2batchquery.Caption := ReplaceString(gj2batchquery.Caption,'%VERSION%',GEOJARRY_VERSION);     
    gj2batchquery_LabelQueryName.Caption := name;
    gj2batchquery_RGroupQueryOption.ItemIndex := 1;
    BatchqueryRGroupOptionOnClick(nil);
    if (gj2batchquery.showmodal = 1) then 
    begin
       case gj2batchquery_RGroupQueryOption.ItemIndex of
       0 : Result := 'QueryDown|' + name;
       1 : Result := 'QueryAdd|' + name;
       2 : Result := 'QuerySub|' + name;
       end;
    end;
end;

procedure GeoJarryLoginOkButtonOnClick(Sender: TObject);
begin
  If ((gj2login_EditUsername.Text <> '') and (gj2login_EditPassword.Text <> '')) 
    then gj2login.ModalResult := 1
    else ShowMessage(_('You must enter username and password.'));
end;

function GeoJarryLoginGet(var username : String; var password : String) : boolean;
begin
    Result := false;
    gj2login.Caption := _('Groundspeak (Geocaching.com) login');
    gj2login_EditUsername.Text := '';
    gj2login_EditPassword.Text := '';
    if (gj2login.showmodal = 1) then
    begin
       username := gj2login_EditUsername.Text;
       password := gj2login_EditPassword.Text;
       Result := true;
    end;
end;

{Add command into batch (format string, ask for user input}
function AddCommandToBatch(name: String) : String;
var
   list : TStringList;
   s,username,password : String;
begin
   case name of
   'GgImport',
   'PendDown',
   'PendClear',
   'GgClearWindow',
   'PendSub',
   'PendOutFndOwn',
   'PendOutDb',
   'PendLoadWindow',
   'PendDownAll',
   'GsLoginReset',
   'GcGetFinds',
   'GgHideImport': begin Result := name; End;
   'GgRunGGP':
      begin
        if OpenDlg(_('Select a GGP plugin'), GEOGET_SCRIPTDIR, 'ofFileMustExist',_('GeoGet GGP plugins') + ' (*.ggp.pas)|*.ggp.pas',s) 
          then Result := name + '|' + GeogetDatadirToDatadirVariable(s)
          else Result := RESULT_INVALID;
      end;
   'PendSave':
      begin
        if SaveDlg(_('Select a GGT list'), GEOGET_DATADIR, '',_('GeoGet GGT lists') + ' (*.ggt)|*.ggt',s) then 
          begin
              if not RegexFind('.ggt$', s) then s := s + '.ggt';
              Result := name + '|' + GeogetDatadirToDatadirVariable(s);
          end
          else Result := RESULT_INVALID;
      end;       
   'GgSetRefPoint':
      begin
         list := TStringList.Create;
         try
          RefPointNames(list);
          if InputComboBox(_('Reference point'), _('Select reference point'), list[0],list,false,s)
           then Result := name + '|' + s
           else Result := RESULT_INVALID;
         finally
          list.free;
         end;
      end;
    'GsLoginChange':
       begin
         if GeoJarryLoginGet(username, password) 
            then Result := name + '|' + username + '|' + password
            else Result := RESULT_INVALID;
       end;
    'GgDbChange':
       begin
         list := TStringList.Create;
         try
          GeoDBNames(list);
          if InputComboBox(_('GeoGet database'), _('Select GeoGet database'), GEOGET_DBName,list,false,s)
           then Result := name + '|' + s
           else Result := RESULT_INVALID;
         finally
          list.free;
         end;
       end; 
   Else Result := RESULT_INVALID;
   End;
end;

{Add query into batch (format string, ask for user input}
function AddQueryToBatch(name: String) : String;
var s : String;
begin
  Result := GetQueryFile(name);
  if DownloadsListings(Result) then
  begin
    Result := BatchqueryGetQueryDownloadOption(name);
  end
  else
    Result := 'QueryDown|' + name;
  
  if gj2batch_BatchListBoxIncluded.Items.IndexOf(Result) <> -1 then
  begin
     s := _('Command "%RESULT%" is already included in batch.');
     s := ReplaceString(s,'%RESULT%',Result);
     ShowMessage(s);
     Result := RESULT_INVALID;
  end;
end;

{Add a command/query into a batch}
procedure BatchButtonAddOnClick(Sender :TObject);
var
   name,res : String;
begin
    case gj2batch_BatchPageControl.TabIndex of
      0: begin res := AddQueryToBatch(gj2batch_BatchListBoxQueries.Items[gj2batch_BatchListBoxQueries.ItemIndex]); end;
      1: begin res := AddCommandToBatch(GeojarryBatchCommandToCommandName(gj2batch_BatchListBoxCommands.Items[gj2batch_BatchListBoxCommands.ItemIndex])); end;
      Else begin ShowMessage(_('Could not recognize command to add into the batch')); Exit; end;
    End;
    if res = RESULT_INVALID 
      then Exit
      else name := res;
            
    gj2batch_BatchListBoxIncluded.Items.Add(name);
    gj2batch_BatchListBoxIncluded.ItemIndex := gj2batch_BatchListBoxIncluded.Items.IndexOf(name);
    gj2batch_BatchButtonSave.Enabled := true;
    BatchListBoxIncludedOnClick(Sender); 
end;

procedure BatchButtonUpOnClick(Sender :TObject);
var
    selected : Integer;
begin
    if gj2batch_BatchListBoxIncluded.ItemIndex <> -1 then selected := gj2batch_BatchListBoxIncluded.ItemIndex;
    gj2batch_BatchListBoxIncluded.Items.Move(selected, selected-1);
    gj2batch_BatchListBoxIncluded.ItemIndex := selected -1;
    BatchListBoxIncludedOnClick(Sender); 
end;

procedure BatchButtonDownOnClick(Sender :TObject);
var
    selected : Integer;
begin
    if gj2batch_BatchListBoxIncluded.ItemIndex <> -1 then selected := gj2batch_BatchListBoxIncluded.ItemIndex;
    gj2batch_BatchListBoxIncluded.Items.Move(selected, selected+1);
    gj2batch_BatchListBoxIncluded.ItemIndex := selected + 1;
    BatchListBoxIncludedOnClick(Sender);                                         
end;

procedure BatchButtonRemoveOnClick(Sender :TObject);
begin
    gj2batch_BatchListBoxIncluded.Items.Delete(gj2batch_BatchListBoxIncluded.ItemIndex);
    BatchListBoxIncludedOnClick(Sender);
    if gj2batch_BatchListBoxIncluded.Items.Count = 0 then gj2batch_BatchButtonSave.Enabled := false;
end;

procedure GeoJarrybatchSetDefaults;
begin
    gj2batch_BatchListBoxCommands.Items := GeojarryBatchAvailableCommands;
    
    gj2batch_BatchPageControl.TabIndex := 0;
    gj2batch_BatchPageControlIncluded.TabIndex := 0;    
    gj2batch_BatchLabelIncluded.Caption := '';
    gj2batch_BatchListBoxIncluded.Items.Clear;
    gj2batch_BatchListBoxQueries.Items := GetFilteredFileList(QUERY_DIR, QUERY_SUFFIX);
    gj2batch_BatchListBoxCommands.ItemIndex := -1;
    gj2batch_BatchLabelAbout.Caption := '';
    gj2batch_BatchMemoDescription.Text := '';
    batchfile := '';
    gj2batch_BatchButtonAdd.Enabled := false;
    gj2batch_BatchButtonRemove.Enabled := false;
    gj2batch_BatchButtonUp.Enabled := false;
    gj2batch_BatchButtonDown.Enabled := false;
        
    gj2batch_BatchListBoxIncluded.Hint := _('List of commands included in this batch');
    gj2batch_BatchListBoxQueries.Hint := _('List of available queries');
    gj2batch_BatchListBoxCommands.Hint := _('List of available commands');
    
end;

{Save current batch to file}
procedure GeoJarrybatchSave(file : String);
var
   list : TStringList;
begin
   list := TStringList.Create;
   try
      list.AddStrings(gj2batch_BatchListBoxIncluded.Items);
      list.Insert(0,'# GEOJARRY_VERSION = ''' + GEOJARRY_VERSION + '''');
      if gj2batch_BatchMemoDescription.Text <> '' 
        then list.Insert(1,'# DESCRIPTION = ''' + WrapCRLF(gj2batch_BatchMemoDescription.Text) + '''');
      list.SaveToFile(file);
   finally
     list.Free;
   end;
end;

{Load query - determine query type and delegate}
function LoadQuery(file : String) : String;
begin
    case GetQueryBase(file) of 
    QUERYBASE_WEB : Result := GeoJarrywebLoadQuery(file);
    QUERYBASE_WM : Result := GeoJarrywmLoadQuery(file);
    QUERYBASE_DB : Result := GeoJarrydbLoadQuery(file);
    else 
      begin
        Result := _('Could not determine query type or file not found:' + CRLF + '%FILE%');
        Result := ReplaceString(Result,'%FILE%',file);
      end;
    End;
end;

{Load and run query - determine query type and delegate}
function LoadAndRunQuery(file : String) : String;
begin
    case GetQueryBase(file) of 
    QUERYBASE_WEB : Result := GeoJarrywebLoadAndRunQuery(file);
    QUERYBASE_WM : Result := GeoJarrywmLoadAndRunQuery(file);
    QUERYBASE_DB : Result := GeoJarrydbLoadAndRunQuery(file);
    else 
      begin
        Result := _('Could not determine query type or file not found:' + CRLF + '%FILE%');
        Result := ReplaceString(Result,'%FILE%',file);
      end;
    End;
end;

{Load and get gc list - determine query type and delegate}
function LoadAndGetGCList(file : String; var gclist : TStringList) : String;
begin
    case GetQueryBase(file) of 
    QUERYBASE_WEB : Result := GeoJarrywebLoadAndGetGCList(file, gclist);
    QUERYBASE_WM : Result := GeoJarrywmLoadAndGetWMList(file, gclist);
    QUERYBASE_DB : Result := GeoJarrydbLoadAndGetGCList(file, gclist);
    else 
      begin
        Result := _('Could not determine query type or file not found:' + CRLF + '%FILE%');
        Result := ReplaceString(Result,'%FILE%',file);
      end;
    End;
end;

{Load run and import query - determine query type and delegate}
function LoadRunAndImportQuery(file : String) : String;
begin
    case GetQueryBase(file) of 
    QUERYBASE_WEB : Result := GeoJarrywebLoadRunAndImportQuery(file);
    QUERYBASE_WM : Result := GeoJarrywmLoadRunAndImportQuery(file);
    QUERYBASE_DB : Result := GeoJarrydbLoadRunAndImportQuery(file);
    else 
      begin
        Result := _('Could not determine query type or file not found:' + CRLF + '%FILE%');
        Result := ReplaceString(Result,'%FILE%',file);
      end;
    End;
end;

{Load query and show - determine query type and delegate}
function LoadQueryAndShow(file : String) : String;
begin
    case GetQueryBase(file) of 
    QUERYBASE_WEB : Result := GeoJarrywebLoadQueryAndShow(file);
    QUERYBASE_WM : Result := GeoJarrywmLoadQueryAndShow(file);
    QUERYBASE_DB : Result := GeoJarrydbLoadQueryAndShow(file);
    else 
      begin
        Result := _('Could not determine query type or file not found:' + CRLF + '%FILE%');
        Result := ReplaceString(Result,'%FILE%',file);
      end;
    End;
end;

{Validate a command included in a batch}
function GeoJarrybatchCommandValidate(command : String) : String;
var
   par,file: String;
   list : TStringList;
begin
   if (pos('|',command) > 0) then //divide into command and parameter
   begin
      par := SeparateRight(command,'|');
      command := SeparateLeft(command,'|');
   end;
   
   Result := _('Unknown command: %COMMAND%') + CRLF;
   Result := ReplaceString(Result,'%COMMAND%',command);
   case command of
   'GgImport',
   'PendDown',
   'PendOutFndOwn',
   'GcGetFinds',
   'PendLoadWindow',
   'PendOutDb',
   'PendDownAll',
   'PendClear',
   'GgClearWindow',
   'GsLoginReset',
   'GgHideImport',
   'PendSub': begin Result := RESULT_VALID end;
   'GgRunGGP':
      begin
        par := DatadirVariableToGeogetDatadir(par);
        if not FileExists(par) 
          then 
            begin
              Result := _('No such file: %FILE%') + CRLF;
              Result := ReplaceString(Result,'%FILE%',par);
            end
          else Result := RESULT_VALID;
      end;
   'PendSave': 
      begin 
        if par <> '' 
          then Result := RESULT_VALID
          else Result := _('No filename specified.');
      end;  
   'GgSetRefPoint':
      begin
          list := TStringList.Create;
          try
            RefPointNames(list);
            if (list.IndexOf(par) = -1)
              then 
                begin
                  Result := _('No such reference point: %REFPOINT%') + CRLF;
                  Result := ReplaceString(Result,'%REFPOINT%',par);
                end
              else Result := RESULT_VALID;
          finally
            list.Free;
          end;
      end;
    'QueryDown':
      begin
          file := GetQueryFile(par);
          if FileExists(file) 
            then Result := LoadQuery(file)
            else 
              begin
                Result := _('No such file: %FILE%')+ CRLF;
                Result := ReplaceString(Result,'%FILE%',par);
              end;
      end;
     'QueryAdd','QuerySub':
      begin
          file := GetQueryFile(par);
          if FileExists(file) then 
            begin 
              if DownloadsListings(file)
                then Result := LoadQuery(file)
                else Result := _('The query %FILE% was changed and can no longer be a part of this batch. To use this command, query must be set to download listings.'); 
            end
          else Result := _('No such file: %FILE%')+ CRLF;
          Result := ReplaceString(Result,'%FILE%',par);
      end;  
    'GsLoginChange':
      begin
         Result := '';
         if SeparateLeft(par,'|') = '' then Result := Result + _('No username specified');
         if SeparateRight(par,'|') = '' then Result := Result + _('No password specified');
         if Result = '' then Result := RESULT_VALID;
      end;
    'GgDbChange':
      begin
          list := TStringList.Create;
          try
            GeoDBNames(list);
            if (list.IndexOf(par) = -1)
              then 
                begin
                  Result := _('No such database: %DB%') + CRLF;
                  Result := ReplaceString(Result,'%DB%',par);
                end
              else Result := RESULT_VALID;
          finally
            list.Free;
          end;      
      end;          
   End;
end;

{Validate a batch}
function GeoJarrybatchValidate(file : String): String;
var
    batchlist : TStringList;
    b : Integer;
    r,s : String;
begin
  Result := '';
  batchlist := GetBatchList(file);
  try
     Result := isBatchCompatible(file);
     if Result = RESULT_VALID then Result := '' else Result := Result + CRLF;

     for b := 0 to batchlist.count -1 do
     begin
       r := GeoJarrybatchCommandValidate(batchlist[b]);
       if r <> RESULT_VALID then 
       begin
          Result:= Result + _('Invalid command: %COMMAND%' + CRLF + 'Error: %ERROR%' + CRLF);
          Result := ReplaceString(Result,'%COMMAND%', batchlist[b]);
          Result := ReplaceString(Result,'%ERROR%', r);
       end;
     end;

    if (Result <> '') 
      then 
        begin
          s := _('This batch is invalid. Errors: ' + CRLF + '%ERRORS%');
          s := ReplaceString(s,'%ERRORS%', Result);
          Result := s;
        end
      else Result := RESULT_VALID;
  finally
    batchlist.free;
  end;
end;

{Load batch into dialog}
function GeoJarrybatchLoad(file : String) : String;
begin
    if FileExists(file) then
    begin
      Result := GeoJarrybatchValidate(file);
      gj2batch_BatchListBoxIncluded.Items := GetBatchList(file);
      gj2batch_BatchMemoDescription.Text := UnwrapCRLF(GetVariableFromBatch('DESCRIPTION',file));
      if gj2batch_BatchMemoDescription.Text = RESULT_INVALID then gj2batch_BatchMemoDescription.Text := '';
      batchfile := file;
      gj2batch_BatchLabelIncluded.Caption := GetBatchName(file);
      if gj2batch_BatchListBoxIncluded.Items.Count > 0 then gj2batch_BatchButtonSave.Enabled := true;
    end else 
          begin
            Result := _('No such file: %FILE%');
            Result := ReplaceString(Result,'%FILE%',file);
          end;
end;

{Show batch dialog}
procedure GeoJarrybatchShow;
begin
    if gj2batch_BatchListBoxQueries.Items.Count = 0 then ShowMessage(_('You have to create at least one query in order to be able to create batches!'))
    else gj2batch.showmodal;
end;

{Load batch and show dialog. If batch is invalid, show message with error}
function GeoJarrybatchLoadAndShow(file : String) : String;
begin
    Result := GeoJarrybatchLoad(file);
    If Result <> RESULT_VALID then ShowMessage(Result);
    GeoJarrybatchShow;
end;

{Show text info about this batch}
function GeoJarrybatchInfo(file : String) : String;
var
  batchlist: TStringList;
  b : Integer;
begin
  Result := GeoJarrybatchValidate(file)
  if Result = RESULT_VALID then
  begin
    Result := UnwrapCRLF(GetVariableFromBatch('DESCRIPTION',file));
    if ((Result = RESULT_INVALID) or (Result = '')) then
    begin
      batchlist := GetBatchList(file);
      try 
        Result := _('This batch combines the following:') + CRLF + CRLF;
        for b := 0 to batchlist.count-1 do
        begin
           Result := Result + batchlist[b];
           If b < batchlist.count-1 then Result := Result + ', ';
        end;
      finally
        batchlist.free;
      end;
    end;
  end;
end;

{Run command included in batch}
function TriggerCommand(command : String) : String;
var
   par,file : String;
   list : TStringList;
begin
   if (pos('|',command) > 0) then //divide into command and parameter
   begin
      par := SeparateRight(command,'|');
      command := SeparateLeft(command,'|');
   end;
   
   Result := _('Could not interpret this command: %COMMAND%') + CRLF;
   Result := ReplaceString(Result,'%COMMAND%',command);
   case command of
   'QueryDown':
      begin                
         Result := LoadAndRunQuery(GetQueryFile(par));
      end;
   'QueryAdd':
      begin
         list := TStringList.Create;
         try
            Result := LoadAndGetGCList(GetQueryFile(par), list);
            if Result = RESULT_VALID then AddToPendingList(list);
         finally
            list.Free;
         end;
      end;
   'QuerySub':
      begin
         list := TStringList.Create;
         try
            Result := LoadAndGetGCList(GetQueryFile(par), list);
            if Result = RESULT_VALID then SubtractFromPendingList(list);
         finally
            list.Free;
         end;
      end;
   'GgImport':
      begin
        GeoJarryGeoImportDir(GetDatadir); EmptyDir(GetDatadir); GeoBusyCaption(''); GeoBusyKind(''); GeoBusyValue('');
        Result := RESULT_VALID;
      End;
   'PendOutFndOwn':
      begin
        SubtractFromPendingList(GetFoundOwnedList);
        Result := RESULT_VALID;
      end;
   'PendOutDb':
      begin
         list := TStringList.Create;
         try
           GEOGET_DB.GetTableStrings('SELECT id FROM geocache;', list);
           SubtractFromPendingList(list);
         finally
           list.Free;
         end;
         Result := RESULT_VALID;
      end;
   'GcGetFinds': 
      begin
         StringToFile(RegexReplace('filename=.+' + CRLF,GetIniHeader,'filename=ZZZZ_DatesOfFinds' + CRLF,false),GetIniFile); //we have to ensure that this file will be imported as the last one
         Result := GeoJarryResultToString(GeoJarryRunExec('"' + GetGeoJarryExe + '" "' + GetIniFile + '" raw "http://www.geocaching.com/my/logs.aspx?s=1"'));
      end;
   'GgHideImport':
      begin
          hideNextImport := true;
          Result := RESULT_VALID;
      end;
   'PendClear':
      begin   
        ClearPendingList; 
        Result := RESULT_VALID;
      end;    
   'PendDown':
      begin
        Result := DownloadWaypointListings(GetPendingList,false);
        ClearPendingList;
      end;
   'PendDownAll':  
      begin
        Result := DownloadWaypointListings(GetPendingList,true);
        ClearPendingList;
      end; 
   'PendLoadWindow':
      begin 
        file := TMP_DIR + 'currentlist.ggt'; 
        if FileExists(file) then DeleteFile(file);
        GetPendingList.SaveToFile(file);
        GeoListFileAdd(file);
        DeleteFile(file);
        Result := RESULT_VALID;
      end;      
   'GgRunGGP':
      begin
        par := DatadirVariableToGeogetDatadir(par);
        if FileExists(par) then
        begin
          GeoCallGGP(par);
          Result := RESULT_VALID;
        end
        else 
          begin
            Result := _('Could not find GeoGet GGP plugin: %FILE%');
            Result := ReplaceString(Result,'%FILE%',par);
          end;
      end;
   'PendSave':
      begin
        if par <> '' then
        begin
          par := DatadirVariableToGeogetDatadir(par);
          GetPendingList.SaveToFile(par);
          Result := RESULT_VALID;
        end
        else Result := _('No GGT list filename specified');
      end; 
   'PendSub':
      begin 
        file := TMP_DIR + 'currentlist.ggt'; 
        if FileExists(file) then DeleteFile(file);
        GetPendingList.SaveToFile(file);
        GeoListFileSub(file);
        DeleteFile(file);
        Result := RESULT_VALID;
      end;   
   'GgSetRefPoint':
      begin
        if RefPointSelect(par) 
          then Result := RESULT_VALID
          else 
            begin
              Result := _('Could not set reference point: %REFPOINT%');
              Result := ReplaceString(Result,'%REFPOINT%',par);
            end;
      end;
   'GgClearWindow':
      begin
         GeoListClear;
         Result := RESULT_VALID;
      end;
   'GsLoginReset':
      begin
         gUsername := gsUsername;
         gPassword := gsPassword;
         Result := RESULT_VALID;
      end;
   'GsLoginChange':
      begin
         gUsername := SeparateLeft(par,'|');
         gPassword := SeparateRight(par,'|');    
         Result := RESULT_VALID;
      end; 
   'GgDbChange':
      begin
         GeoDBSelect(par);
         Result := RESULT_VALID;
      end;        
   End;
end;

{Execute this batch. Does the hard work}
function ExecuteBatch(file: String) : String;
var
    batchlist : TStringList;
    com,par : String;
    b : Integer;
begin
  Result := '';
  batchlist := GetBatchList(file);
  try
    for b := 0 to batchlist.count-1 do //go through the commands
    begin
       //Busy
       GeoBusyCaption(_('GeoJarry'));
       com := batchlist[b];
       if (pos('|',com) > 0) then 
          begin
            par := SeparateRight(com,'|'); 
              if (pos('|',par) > 0) then par := SeparateLeft(par,'|');
            com := SeparateLeft(com,'|');
            com := GeojarryBatchCommandIdToCommandName(com) + ' (' + par + ')';
          end
       else com := GeojarryBatchCommandIdToCommandName(com);
       GeoBusyKind(com);
       GeoBusyProgress(b+1,batchlist.count);
       
       //call the command
       Result := TriggerCommand(batchlist[b]); 
    
       if ((b = batchlist.count-1) and (Result = RESULT_VALID) and (GetPendingList.Count > 0)) //in the last round, when pending list is not empty...
        then Result := DownloadWaypointListings(GetPendingList,false); //...download pending waypoints
      
       if Result <> RESULT_VALID then Exit; //if something goes wrong, don't continue and return an error
    end;
    GeoBusyCaption('');
    GeoBusyKind('');
    GeoBusyValue('');
  finally
    batchlist.free;
  end; 
end;

{Run this batch. Prepares important stuff and executes the batch}
function RunBatch(file : String) : String;
begin
  if FileExists(file) then
  begin
    Result := GeoJarrybatchValidate(file);
    if Result = RESULT_VALID then //we can download it
    begin
      EmptyDir(GetDatadir);
      GeoJarryGeoImportBegin;
      Result := ExecuteBatch(file);
      if Result = RESULT_VALID then //we can import it
      begin
        GeoJarryGeoImportDir(GetDatadir);
        GeoJarryGeoImportEnd;
      end;
    end;
  end
  else 
    begin
      Result := _('No such file: %FILE%');
      Result := ReplaceString(Result,'%FILE%',file);
    end;
end;

{Run this query. Delegates the hard work}
function RunQuery(file : String) : String;
var s : String;
begin
  if FileExists(file) 
    then Result := LoadRunAndImportQuery(file)
    else 
      begin
        s := _('No such file: %FILE%');
        s := ReplaceString(s,'%FILE%',file);
        ShowMessage(s);
      end;
end; 

procedure BatchButtonSaveOnClick(Sender :TObject);
var
  file,tmp,res : String;
begin
  tmp := TMP_DIR + 'batch.tmp'; //as we can only validate batchfiles, we have to save it
  GeoJarrybatchSave(tmp);
  res := GeoJarrybatchValidate(tmp);
  if res = RESULT_VALID then //its ok, save this batch
  begin
    file := GetFile(BATCH_DIR, BATCH_SUFFIX, _('Batch'), GetBatchName(batchfile));
    if file <> '' then 
    begin
      GeoJarrybatchSave(file);
      batchfile := '';
      gj2batch.close;
    end;                     
  end else ShowMessage(res);
  DeleteFile(tmp); //.. and then clean it
end;

procedure BatchButtonCancelOnClick(Sender :TObject);
begin
  batchfile := '';
end;

procedure GeoJarrybatchSetActions;
begin
    gj2batch.Caption := _('GeoJarry %VERSION% - batch editor');
    gj2batch.Caption := ReplaceString(gj2batch.Caption,'%VERSION%',GEOJARRY_VERSION); 
    
    gj2batch_BatchListBoxQueries.Items := GetFilteredFileList(QUERY_DIR, QUERY_SUFFIX);
    
    gj2batch_BatchListBoxIncluded.OnClick := @BatchListBoxIncludedOnClick;
    gj2batch_BatchListBoxQueries.OnClick := @BatchListBoxQueriesOnClick;
    gj2batch_BatchListBoxCommands.OnClick := @BatchListBoxCommandsOnClick;
    
    gj2batch_BatchPageControl.onChange := @BatchPageControlOnChange;
    
    gj2batch_BatchButtonAdd.OnClick := @BatchButtonAddOnClick;
    gj2batch_BatchButtonUp.OnClick := @BatchButtonUpOnClick;
    gj2batch_BatchButtonDown.OnClick := @BatchButtonDownOnClick;
    gj2batch_BatchButtonRemove.OnClick := @BatchButtonRemoveOnClick;
    
    gj2batch_BatchButtonSave.OnClick := @BatchButtonSaveOnClick;
    gj2batch_BatchButtonCancel.OnClick := @BatchButtonCancelOnClick;
    
    gj2batchquery_RGroupQueryOption.OnClick := @BatchqueryRGroupOptionOnClick;
    gj2login_OkButton.OnClick := @GeoJarryLoginOkButtonOnClick;
end;
