{Translate filter name into file name}
function GetFilterFile(name : String) : String;
begin
   Result := GEOGET_DATADIR + '\' + name + '.ggf';
end;

function GeoJarrydbFilterValidate : String;
var
  gjdbResult,res : String;
begin
  Result := '';
  if (gj2db_FilterComboBox.Text = '') then Result := Result + _('You have to select a filter. If you want to apply the rules to the whole database, just create and use a nonrestrictive filter. (No "Filter" selected or file not found)') + CRLF else
  if (not FileExists(GetFilterFile(gj2db_FilterComboBox.Text))) then 
  begin
    res := _('There is no such filter: %FILTER%. (File not found)');
    res := ReplaceString(res,'%FILTER%',gj2db_FilterComboBox.Text);
    Result := Result + res + CRLF; 
  end;

  gjdbResult := GeoJarrydbAllValidate;
  if gjdbResult <> RESULT_VALID then Result := Result + gjdbResult;
  
  if Result <> '' Then 
      Result := Result + RESULT_INVALID
  else Result := RESULT_VALID;
end;

function GeoJarrydbFilterLoadSettings(file: String) : String;
var
  iniF: TIniFile;
begin
  Result := GeoJarrydbAllLoadDownloadSettings(file);
  if FileExists(file) and (Result = RESULT_VALID) then
  begin
    iniF := TIniFile.Create(file);
    try
      gj2db_FilterComboBox.ItemIndex      := gj2db_FilterComboBox.Items.IndexOf(iniF.readString('Query', 'Filter', ''));
      
      Result := GeoJarrydbFilterValidate;
    finally
      iniF.Free;
    end;
  end;
end;

function GeoJarrydbFilterSaveSettings(file: String) : String;
var
  iniF: TIniFile;
begin
  Result := GeoJarrydbFilterValidate;
  if Result = RESULT_VALID then
  begin
    iniF := TIniFile.Create(file);
    try
       GeoJarrydbAllSaveQuery(file);
       iniF.writeString('Query', 'Filter', gj2db_FilterComboBox.Text);
       
       Result := GeoJarrydbAllSaveDownloadSettings(file);
    finally
      iniF.Free;
    end;
  end;
end;

{Get stringlist of waypoint codes}
function GeoJarrydbFilterGetCacheList(var list: TStringList) : String;
var
  all : TStringList;
  geo : TGeo;
  q,busy : String;
  n,max: Integer;
begin
  Result := '';
  GeoBusyCaption(_('GeoJarry'));
  busy := _('Getting list (filter: %FILTER%)');
  busy := ReplaceString(busy,'%FILTER%',gj2db_FilterComboBox.Text);
  GeoBusyKind(busy);
  List.Clear;
  GeoFilterLoadFile(GetFilterFile(gj2db_FilterComboBox.Text));
  all := TStringList.Create;
  geo := TGeo.Create;
  try  
      q := 'SELECT id FROM geocache;'; //get all waypoints in the database     
      GEOGET_DB.GetTableStrings(q, all);
      max := all.count;
      for n := 0 to max-1 do
      begin
        if GeoBusyTest then begin ShowMessage(_('Canceled')); GeoAbort; end;
    		GeoBusyProgress(n+1,max);        
    		if not (IsValidGc(all[n]) or IsValidWm(all[n])) then  //skip non-(GC or WM) records
              continue;
        
        geo.LoadByGC(all[n]);
        if ((geo.isFiltered) and (not geo.isEmptyCoord)) then list.Add(all[n]); //determine whether the waypoint passes through the filter (we don't want geocaches without coordinates - probably PM only)
      End;
      If Result = '' then Result := RESULT_VALID;
  except
   
  finally
     GeoBusyCaption('');
     GeoBusyKind('');
     GeoBusyValue('');
     all.Free;
     geo.Free;
  end; 
end;

function GeoJarrydbFilterGetInfo : String;
begin
    Result := _('This query will download listings for waypoints selected from GeoGet database by filter "%FILTER%".') + ' ';
    Result := ReplaceString(Result,'%FILTER%',gj2db_FilterComboBox.Text);
end;

procedure GeoJarrydbFilterSetDefaults;
begin
  gj2db_FilterComboBox.Items := GetFilteredFileList(GEOGET_DATADIR + '\', 'ggf');
  
  gj2db_FilterComboBox.ItemIndex := -1;
end;

procedure GeoJarrydbFilterSetActions;
begin
  GeoJarrydbFilterSetDefaults;
end;
