var
  dbdownloaded : boolean; //was the query downloaded?
  dbqueryfile : string; //what file was the query loaded from?

{Validate the common part of the query}     
function GeoJarrydbAllValidate : String;
begin
  Result := '';
  if (gj2db_AllRGroupFound.ItemIndex = -1) then Result := Result + _('You have to select how to handle the found or the owned waypoints. (Unknown "Found or owned waypoints" option)') + CRLF;  
  
  if Result = '' Then Result := RESULT_VALID;   
end;

{Load download-related information about this query}
function GeoJarrydbAllLoadDownloadSettings(file: String) : String;
var
  iniF: TIniFile;
  res : String;
begin
  Result := '';
  iniF := TIniFile.Create(file);
  try
    Result := isQueryCompatible(file);
    if Result = RESULT_VALID then Result := '' else Result := Result + CRLF;
    gj2db_AllRGroupFound.ItemIndex   := iniF.readInteger('Query', 'FilterOutFinds', DEFAULT_RGROUP);
    
    res := GeoJarrydbAllValidate;
    if res = RESULT_VALID then res := '';
    
    if ((Result <> '') or (res <> '')) 
      then Result := Result + res
      else Result := RESULT_VALID;
  finally
    iniF.Free;
  end;
end;

{Save general information about this query}
procedure GeoJarrydbAllSaveQuery(file: String);
var
  iniF: TIniFile;
begin
  iniF := TIniFile.Create(file);
  try
    iniF.writeString('GeoJarry','Version', GEOJARRY_VERSION);
    iniF.writeString('Query', 'QueryBase', QUERYBASE_DB);
    iniF.writeString('Query', 'QueryType', gj2db_AllPageControl.ActivePage.Name);
    finally
    iniF.Free;
  end;
end;

{Save download-related information about this query}
function GeoJarrydbAllSaveDownloadSettings(file: String) : String;
var
  iniF: TIniFile;
begin
  Result := GeoJarrydbAllValidate;
  if Result = RESULT_VALID then
  begin
  iniF := TIniFile.Create(file);
    try
      iniF.writeInteger('Query', 'FilterOutFinds', gj2db_AllRGroupFound.ItemIndex);
    finally
      iniF.Free;
    end;
  end;
end;

{$INCLUDE gj2db.current.lib.pas}
{$INCLUDE gj2db.filter.lib.pas}
{$INCLUDE gj2db.list.lib.pas}
{$INCLUDE gj2db.sql.lib.pas}

{Set functions for downloading, validating etc. (determine from current tab)}
procedure GeoJarrydbSetFunctions;
begin
    Case gj2db_AllPageControl.ActivePage.Name of
    'CurrentTab' :
    begin
        ValidateF := @GeoJarrydbCurrentValidate;
        GClistF   := @GeoJarrydbCurrentGetCacheList;
        InfoF     := @GeoJarrydbCurrentGetInfo;
        SaveF     := @GeoJarrydbCurrentSaveSettings;
        LoadF     := @GeoJarrydbCurrentLoadSettings;
    end;
    'FilterTab' : 
    begin
        ValidateF := @GeoJarrydbFilterValidate;
        GClistF   := @GeoJarrydbFilterGetCacheList;
        InfoF     := @GeoJarrydbFilterGetInfo;
        SaveF     := @GeoJarrydbFilterSaveSettings;
        LoadF     := @GeoJarrydbFilterLoadSettings;
    end;
    'ListTab' : 
    begin
        ValidateF := @GeoJarrydbListValidate;
        GClistF   := @GeoJarrydbListGetCacheList;
        InfoF     := @GeoJarrydbListGetInfo;
        SaveF     := @GeoJarrydbListSaveSettings;
        LoadF     := @GeoJarrydbListLoadSettings;
    end;
    'SqlTab' : 
    begin
        ValidateF := @GeoJarrydbSqlValidate;
        GClistF   := @GeoJarrydbSqlGetCacheList;
        InfoF     := @GeoJarrydbSqlGetInfo;
        SaveF     := @GeoJarrydbSqlSaveSettings;
        LoadF     := @GeoJarrydbSqlLoadSettings;
    end;   
  end; 
end;

{Set default values into the form}
procedure GeoJarrydbSetDefaults;
begin
   GeoJarrydbCurrentSetDefaults;
   GeoJarrydbFilterSetDefaults;
   GeoJarrydbListSetDefaults;
   GeoJarrydbSqlSetDefaults;
   
   gj2db_AllPageControl.TabIndex := 0; 
   gj2db_AllRGroupFound.ItemIndex := 0;
   
   dbqueryfile := '';
end;

{Get stringlist of waypoint codes}
function GeoJarrydbGcList(var list : TStringList) : String;
begin
    Result := GclistF(list);
    
    //Remove non-(GC or WM) codes
    list := CodeListRemoveInvalid(list);
    
    if gj2db_AllRGroupFound.ItemIndex = 1 then
        SubtractFromList(list, GetFoundOwnedList);
end;

{Load the query into the dialog}
function GeoJarrydbLoadQuery(file : String) : String;
var
  iniF: TIniFile;
  tab : Integer;
  
begin
  Result := '';
  GeoJarrydbSetDefaults;
  iniF := TIniFile.Create(file);
  try
      tab := TabNameToTabIndex(iniF.readString('Query', 'QueryType', ''), gj2db_AllPageControl);
      if tab <> -1 then 
      begin  
        gj2db_AllPageControl.TabIndex := tab;
        GeoJarrydbSetFunctions();
        Result := LoadF(file);
        dbqueryfile := file;
      end
      else 
        begin
          Result := _('Unable to determine query type: %FILE%');
          Result := ReplaceString(Result,'%FILE%',file);
        end;
  finally
      iniF.Free;
  end;
end;

{Show information about the query}       
function GeoJarrydbQueryInfo(file : String) : String;
var s : String;
begin
  Result := GeoJarrydbLoadQuery(file);
  if Result = RESULT_VALID then
  begin
    Result := InfoF();
    if (gj2db_AllRGroupFound.ItemIndex = 1) then Result := Result + _('Waypoints marked in GeoGet database as found or owned will not be downloaded.') + ' ';
  end
  else 
    begin
      s := _('This query is invalid. Errors: ' + CRLF + '%ERRORS%');
      s := ReplaceString(s,'%ERRORS%',Result);
      Result := s;
    end;
end;    

{Run this query}
function GeoJarrydbRunQuery: String;
var
  list : TStringList;
begin
  NewGjFilename;
  GeoJarrydbSetFunctions;
  
  Result := ValidateF();
  if Result = RESULT_VALID then 
  begin
    list := TStringList.Create
    try
      GeoJarrydbGcList(list)
      Result := DownloadWaypointListings(list,false);
    finally
      list.Free;
    end;
  end;
end;

{Run and import this query}
function GeoJarrydbRunAndImportQuery: String;
begin
  EmptyDir(GetDatadir);
  Result := GeoJarrydbRunQuery;
  if Result = RESULT_VALID then
  begin
     GeoJarryGeoImportBegin;
     GeoJarryGeoImportDir(GetDatadir);
     GeoJarryGeoImportEnd;
  end;
end;

{Load and run this query}
function GeoJarrydbLoadAndRunQuery(file : String) : String;
begin
    Result := GeoJarrydbLoadQuery(file); 
    if Result = RESULT_VALID then Result := GeoJarrydbRunQuery;
end;

{Load query and get list of waypoint codes}
function GeoJarrydbLoadAndGetGCList(file : String; var gclist : TStringList) : String;
begin
    Result := GeoJarrydbLoadQuery(file);
    if Result = RESULT_VALID then Result := GeoJarrydbGcList(gclist);
end;

{Load run and import this query}
function GeoJarrydbLoadRunAndImportQuery(file : String) : String;
begin
    Result := GeoJarrydbLoadQuery(file); 
    if Result = RESULT_VALID then Result := GeoJarrydbRunAndImportQuery;  
end; 

{Show me the dialog}
function GeoJarrydbShow : String;
begin
    Result := '';
    dbdownloaded := false;
    gj2db.showmodal;
    if dbdownloaded then Result := RESULT_DOWNLOADED;   
end;

{Load query and show in the dialog}
function GeoJarrydbLoadQueryAndShow(file : String) : String;
var s : String;
begin
    Result := GeoJarrydbLoadQuery(file);
    If Result <> RESULT_VALID then
    begin
      s := _('This query is invalid. Errors: ' + CRLF + '%ERRORS%');
      s := ReplaceString(s,'%ERRORS%',Result);
      ShowMessage(s);
    end;
    Result := GeoJarrydbShow;
end;

procedure GeoJarrydbAllButtonSaveOnClick(Sender :TObject);
var
  file,res : String;
begin
  GeoJarrydbSetFunctions;
  res := ValidateF();
  if res = RESULT_VALID then
  begin
    file := GetFile(QUERY_DIR, QUERY_SUFFIX, _('Query'), GetQueryName(dbqueryfile));
    if file <> '' then SaveF(file);
  end
  else ShowMessage(res);
end;

procedure GeoJarrydbAllButtonGgtOnClick(Sender :TObject);
var
  file,res,s : String;
  list : TStringList;
begin
  GeoJarrydbSetFunctions;
  res := ValidateF();
  if res = RESULT_VALID then
  begin
    if SaveDlg(_('Select a GGT list'), GEOGET_DATADIR, 'ofFileMustExist',_('GeoGet GGT lists') + ' (*.ggt)|*.ggt',file) then 
    begin
      if not RegexFind('.ggt$', file) then file := file + '.ggt';
      s := _('%FILE% already exists. Would you like to overwrite?');
      s := ReplaceString(s,'%FILE%',file);
      if FileExists(file) then if (MessBox(s, _('List file'),4) <> 6)then  Exit;
      
      gj2db.hide;
      list := TStringList.Create;
      try
        GeoJarrydbGcList(list);
        list.saveToFile(file);
      finally
        list.Free;
      end;  
      gj2db.show;
    end;
  end
  else ShowMessage(res);
end;

procedure GeoJarrydbAllButtonDownloadNowOnClick(Sender :TObject);
var
  res: String;
begin
    gj2db.hide;
    res := GeoJarrydbRunAndImportQuery;
    if res <> RESULT_VALID then 
    begin
       ShowMessage(res);
       gj2db.show;
    end
    else
    begin
       dbdownloaded := true;
       gj2db.close;
    end;
end;   

procedure GeoJarrydbAllPageControlOnChange(Sender :TObject);
begin
   GeoJarrydbSetFunctions;
end;

procedure GeoJarrydbSetActions;
begin
  gj2db.Caption := _('GeoJarry %VERSION% - GeoGet database based query');
  gj2db.Caption := ReplaceString(gj2db.Caption,'%VERSION%',GEOJARRY_VERSION); 

  gj2db_AllButtonSave.OnClick := @GeoJarrydbAllButtonSaveOnClick;
  gj2db_AllButtonDownloadNow.OnClick := @GeoJarrydbAllButtonDownloadNowOnClick;
  gj2db_AllPageControl.onChange := @GeoJarrydbAllPageControlOnChange; 
  gj2db_AllButtonGgt.OnClick := @GeoJarrydbAllButtonGgtOnClick;
  
  GeoJarrydbCurrentSetActions;
  GeoJarrydbFilterSetActions;
  GeoJarrydbListSetActions;
  GeoJarrydbSqlSetActions; 
end; 