{Save list file into the form}
procedure SetListFile(list : String);
var
   hint : String;
begin
   list := GeogetDatadirToDatadirVariable(list);
   gj2db_ListEditName.Text := list;
   if list <> '' then hint := CRLF + list;
   gj2db_ListGroupBox.Hint := _('Download listings for all waypoints contained in the selected list.') + hint;
   if list <> '' then gj2db_ListSButtonClear.Visible := true else gj2db_ListSButtonClear.Visible := false;
end;

procedure GeoJarrydbListSButtonClearOnClick(sender: TObject);
begin
   SetListFile('');
end;

procedure GeoJarrydbListButtonBrowseOnClick(sender: TObject);
var
  list : String;
begin
  if OpenDlg(_('Select a GGT list'), GEOGET_DATADIR, 'ofFileMustExist',_('GeoGet GGT lists')+ ' (*.ggt)|*.ggt', list) then SetListFile(list);
end;


function GeoJarrydbListValidate : String;
var
  gjdbResult,r : String;
begin
  Result := '';
  if (gj2db_ListEditName.Text = '') then Result := Result + _('You have to select a list. (No "list" selected)') + CRLF else
  if not FileExists(DatadirVariableToGeogetDatadir(gj2db_ListEditName.Text)) then 
  begin
    r := _('There is no such list: %LIST%.');
    r := ReplaceString(r,'%LIST%',DatadirVariableToGeogetDatadir(gj2db_ListEditName.Text)); 
    Result := Result + r + CRLF; 
  end;
  
  gjdbResult := GeoJarrydbAllValidate;
  if gjdbResult <> RESULT_VALID then Result := Result + gjdbResult;
   
  if Result <> '' Then 
      Result := Result + RESULT_INVALID
  else Result := RESULT_VALID;
end;

function GeoJarrydbListLoadSettings(file: String) : String;
var
  iniF: TIniFile;
begin
  Result := GeoJarrydbAllLoadDownloadSettings(file);
  if FileExists(file) and (Result = RESULT_VALID) then
  begin
    iniF := TIniFile.Create(file);
    try
      SetListFile(iniF.readString('Query', 'List', ''));
      Result := GeoJarrydbListValidate;
    finally
      iniF.Free;
    end;
  end;
end;

function GeoJarrydbListSaveSettings(file: String) : String;
var
  iniF: TIniFile;
begin
  Result := GeoJarrydbListValidate;
  if Result = RESULT_VALID then
  begin
    iniF := TIniFile.Create(file);
    try
       GeoJarrydbAllSaveQuery(file);
       iniF.writeString('Query', 'List', gj2db_ListEditName.Text);
       Result := GeoJarrydbAllSaveDownloadSettings(file);
    finally
      iniF.Free;
    end;
  end;
end;

{Get stringlist of waypoint codes}
function GeoJarrydbListGetCacheList(var list: TStringList) : String;
begin
   list.clear;
   list.LoadFromFile(DatadirVariableToGeogetDatadir(gj2db_ListEditName.Text)); //it is a waypoint list already
   Result := RESULT_VALID;
end;

function GeoJarrydbListGetInfo : String;
begin
    Result := _('This query will download listings for all waypoints saved in the given GGT list.') + ' ';
end;  

procedure GeoJarrydbListSetDefaults;
begin
  SetListFile('');
end;

procedure GeoJarrydbListSetActions;
begin
  gj2db_ListButtonBrowse.onClick := @GeoJarrydbListButtonBrowseOnClick;
  gj2db_ListSButtonClear.onClick:= @GeoJarrydbListSButtonClearOnClick;
end;
