{Validate the command. We don't want users to destroy their DB, so we deny some SQL commands}
function SqlCommandValidate(Command : String) : String;
begin
  Result := '';
  If not RegexFind('(?i)^\s*SELECT', Command) then Result := Result + _('SQL command has to start with ''SELECT''') + CRLF; 
  If not RegexFind(';\s*$', Command) then Result := Result + _('SQL command has to end with '';''') + CRLF;
  If RegexFind('(?i)((\s|^)(ABORT|ADD|AFTER|ALTER|ANALYZE|ATTACH|'+
                    'AUTOINCREMENT|BEFORE|BEGIN|CASCADE|COLUMN|'+
                    'COMMIT|CONFLICT|CONSTRAINT|CREATE|DATABASE|'+
                    'DEFAULT|DEFERRABLE|DEFERRED|DELETE|DETACH|'+
                    'DROP|EACH|EXCLUSIVE|EXPLAIN|FAIL|FOR|'+
                    'FOREIGN|FULL|CHECK|IF|IGNORE|IMMEDIATE|'+
                    'INDEX|INITIALLY|INSERT|INSTEAD|INTO|KEY|'+
                    'OF|PLAN|PRAGMA|PRIMARY|QUERY|RAISE|'+
                    'REFERENCES|REINDEX|RELEASE|RENAME|REPLACE|'+
                    'RESTRICT|ROLLBACK|ROW|SAVEPOINT|SET|TABLE|'+
                    'TEMP|TEMPORARY|TO|TRANSACTION|TRIGGER|UNIQUE|'+
                    'UPDATE|VACUUM|VALUES|VIEW|VIRTUAL)(\s|;|$))+',command) then Result := Result + _('You are trying to use a forbidden SQL command.') + CRLF;
  If Result = '' then Result := RESULT_VALID;
end;

{Substitute available variables to real data}
function GeoJarrydbSqlSubstituteVariables(sql: String) : String;
begin
  Result := sql;
  Result := ReplaceString(sql,'%GEOGET_OWNER%',GEOGET_OWNER);
end;

function GeoJarrydbSqlGetSQL() : String;
begin
  Result := GeoJarrydbSqlSubstituteVariables(gj2db_SqlMemo.Text);
end;

function GeoJarrydbSqlValidate : String;
var
  s1,s2 : String;
begin  
  Result := '';
  s1 := SqlCommandValidate(GeoJarrydbSqlGetSQL);
  if s1 <> RESULT_VALID then Result := Result + s1;

  s2 := GeoJarrydbAllValidate;
  if s2 <> RESULT_VALID then Result := Result + s2;
  
  if Result = '' Then Result := RESULT_VALID 
    else Result := Result + RESULT_INVALID;
end;  

function GeoJarrydbSqlLoadSettings(file: String) : String;
var
  iniF: TIniFile;
begin
  Result := GeoJarrydbAllLoadDownloadSettings(file);
  if FileExists(file) and (Result = RESULT_VALID) then
  begin
    iniF := TIniFile.Create(file);
    try
      gj2db_SqlMemo.Text        := UnwrapCRLF(iniF.readString('Query', 'SqlQuery', ''));
 
      Result := GeoJarrydbSqlValidate;
    finally
      iniF.Free;
    end;
  end;
end;

function GeoJarrydbSqlSaveSettings(file: String) : String;
var
  iniF: TIniFile;
begin
  Result := GeoJarrydbSqlValidate;
  if Result = RESULT_VALID then
  begin
    iniF := TIniFile.Create(file);
    try
      GeoJarrydbAllSaveQuery(file);
      iniF.writeString('Query', 'SqlQuery', WrapCRLF(gj2db_SqlMemo.Text));
    
      Result := GeoJarrydbAllSaveDownloadSettings(file);
    finally
      iniF.Free;
    end;
  end;
end;

{Get stringlist of waypoint codes}        
function GeoJarrydbSqlGetCacheList(var list: TStringList) : String;
var s : String;
begin
   Result := '';
   list.Clear;
   if SqlCommandValidate(GeoJarrydbSqlGetSQL) = RESULT_VALID then //passes through rough check
   begin
     try
        GEOGET_DB.GetTableStrings(GeoJarrydbSqlGetSQL, list);
        If CodeListValidate(list) <> RESULT_VALID then //did the query return valid list of geocache/waymark codes?
        begin
           List.Clear;
           Result := RESULT_INVALID;
        end else Result := RESULT_VALID;   
     except //user wrote wrong SQL query
        if exceptiontype = erException then 
        begin
          s := _('Error occured while executing your SQL query:' + CRLF + CRLF + '%ERROR%');
          s := ReplaceString(s,'%ERROR%',ExceptionToString(ExceptionType, ExceptionParam));
          ShowMessage(s)
        end
        else RaiseLastException;
     end;
   end;
end;

procedure GeoJarrydbSqlButtonIOLoadOnClick(Sender: TObject);
var
  file : String;
begin
  if OpenDlg(_('Select a SQL file'), SQL_DIR , 'ofFileMustExist',_('GeoGet SQL queries')+' (*.' + SQL_SUFFIX + ')|*.' + SQL_SUFFIX,file) then 
  begin
    gj2db_SqlMemo.Text := FileToString(file);
  end;
end;

procedure GeoJarrydbSqlButtonIOSaveOnClick(Sender: TObject);
var
  file,res,s : String;
begin
  res := SqlCommandValidate(GeoJarrydbSqlGetSQL);
  if res = RESULT_VALID then
  begin
    if SaveDlg(_('Select a SQL file'), SQL_DIR , '',_('GeoGet SQL queries') + ' (*.' + SQL_SUFFIX + ')|*.' + SQL_SUFFIX,file) then 
    begin
      if not RegexFind('.' + SQL_SUFFIX + '$', file) then file := file + '.' + SQL_SUFFIX;
      s := _('%FILE% already exists. Would you like to overwrite?');
      s := ReplaceString(s,'%FILE%',file);
      if FileExists(file) then if (MessBox(s, _('SQL file'),4) <> 6)then  Exit;
      StringToFile(gj2db_SqlMemo.Text, file);
    end;
  end else ShowMessage(res);
end;

{Get stringlist of waypoint codes and load it into GeoGet window}
procedure GeoJarrydbSqlButtonPrevWinOnClick(Sender: TObject);
var
  tmp,res : String;
  list : TStringList;
begin
  res := SqlCommandValidate(GeoJarrydbSqlGetSQL); 
  if res = RESULT_VALID then
  begin
    list := TStringList.Create;
    try
      res := GeoJarrydbSqlGetCacheList(list);
      if ((list.count > 0) and (res = RESULT_VALID)) then
      begin
        StringToFile(gj2db_SqlMemo.Text,SQL_DIR + 'LastPreviewed.sql'); //save for convenience of users
        gj2db.hide;
        tmp := TMP_DIR + 'currentlist.ggt';
        if FileExists(tmp) then DeleteFile(tmp);
        list.SaveToFile(tmp);
        GeoListClear;
        GeoListFileAdd(tmp);
        dbdownloaded := true;
        gj2db.close;
        DeleteFile(tmp);
      end
      else ShowMessage(_('SQL query returned 0 rows containing geocache/waymark codes.'));
    finally
      list.Free;
    end;
  end
  else ShowMessage(res);
end; 

{Get stringlist of waypoint codes and save it as ggt list}
procedure GeoJarrydbSqlButtonPrevListOnClick(Sender: TObject);
var
  file,res,s : String;
  list : TStringList;
begin
  res := SqlCommandValidate(GeoJarrydbSqlGetSQL); 
  if res = RESULT_VALID then
  begin
    list := TStringList.Create;
    try
      res := GeoJarrydbSqlGetCacheList(list);
      if ((list.count > 0) and (res = RESULT_VALID)) then
      begin
        if SaveDlg(_('Select a GGT list'), GEOGET_DATADIR, 'ofFileMustExist',_('GeoGet GGT lists') + ' (*.ggt)|*.ggt',file) then 
        begin
          if not RegexFind('.ggt$', file) then file := file + '.ggt';
          s := _('%FILE% already exists. Would you like to overwrite?');
          s := ReplaceString(s,'%FILE%',file);
          if FileExists(file) then if (MessBox(s, _('List file'),4) <> 6)then  Exit;
        
          StringToFile(gj2db_SqlMemo.Text,SQL_DIR + 'LastPreviewed.sql');
          list.saveToFile(file); 
        end;
      end
      else ShowMessage(_('SQL query returned 0 rows containing geocache/waymark codes.'));
    finally
      list.Free;
    end;
  end
  else ShowMessage(res);
end;

function GeoJarrydbSqlGetInfo : String;
begin
    Result := _('This query will download listings for all waypoints selected from GeoGet database by the given SQL query.') + ' ';
end;

procedure GeoJarrydbSqlSetDefaults;
begin
   gj2db_SqlMemo.Text := '';
end;

procedure GeoJarrydbSqlSetActions;
begin
   gj2db_SqlButtonIOLoad.onClick := @GeoJarrydbSqlButtonIOLoadOnClick;
   gj2db_SqlButtonIOSave.onClick := @GeoJarrydbSqlButtonIOSaveOnClick;
   gj2db_SqlButtonPrevWin.onClick := @GeoJarrydbSqlButtonPrevWinOnClick;
   gj2db_SqlButtonPrevList.onClick := @GeoJarrydbSqlButtonPrevListOnClick;
end;
