var
  pluginfile,name,originalfile,icon : String; //variables containing information about loaded file, icon etc.

{Extract icon string from GeoGet plugin macro (only simple, might be problematic if used on complicated files)}  
function GeoJarrypluginExtractIcon(file : String): String;
begin
  Result := FileToString(file);
  Result := RegexReplace('^.*DecodeBase64\(''',Result,'',false);
  Result := RegexReplace('''\).*$',Result,'',false);
end;

{Load icon string and return a bitmap}  
function GeoJarrypluginIconStringToBitmap(icon : String) : TBitMap;
var
  stream : TMemoryStream;
  ics : String;
begin
  stream := TMemoryStream.Create;
  try
    ics := DecodeBase64(icon);
    stream.Write(ics, Length(ics));
    stream.Position := 0;
    Result := TBitMap.Create;
    Result.LoadFromStream(stream);
  finally
    stream.free;
  end; 
end;

{Get icon string from bitmap}
function GeoJarrypluginBitmapToIconString(file : String) : String;
begin
   Result := EncodeBase64(FileToString(file));
end;

{Create GeoGet plugin from currently loaded file}
procedure GeoJarrypluginCreate;
var
  s : String;
begin
  if ((pluginfile <> '') and (name <> '') and (originalfile <> '')) then
  begin
    s := '{$include gj2.lib.uses.pas}' + CRLF +
         '{$include gj2.lib.pas}' + CRLF + CRLF + 
         '//Automatically created GeoGet plugin. Do not edit manually' + CRLF + 
         '//Created by GeoJarry version ' + GEOJARRY_VERSION + CRLF + CRLF +
         'function PluginCaption: String;' + CRLF +
         '  begin Result := ''' + Name + '''; end;' + CRLF + CRLF +
         'function PluginHint: String;' + CRLF + 
         '  begin Result := ''' + Name + ' (GeoJarry)''; end;' + CRLF + CRLF + 
         'function PluginIcon: String;' + CRLF + 
         '  begin Result := DecodeBase64(''' + icon + '''); end;' + CRLF + CRLF +          
         'function PluginFlags: String;' + CRLF + 
         '  begin Result := ''notrans''; end;' + CRLF + CRLF + 
         'procedure PluginStart;' + CRLF + 
         '  begin Init; ';
         if isBatchFile(originalfile) then s := s + 'PluginRunBatch(''' + name + ''');'
         else if isQueryFile(originalfile) then s := s + 'PluginRunQuery(''' + name + ''');';
         s := s + ' Cleanup; end;' + CRLF;  

    StringToFile(s,pluginfile);
  end;
end;

{Load file into dialog, init variables}
procedure GeoJarrypluginSet(file : String);
var
   fSuffix,s : String;
begin     
   gj2plugin_LabelName.Caption := '';
   
   if isBatchFile(file) then
   begin
      Name := GetBatchName(file);
      fSuffix := BATCH_PLUGINSUFFIX;
   end
   else if isQueryFile(file) then
   begin
      Name := GetQueryName(file);
      fSuffix := QUERY_PLUGINSUFFIX;
   end
   else 
   begin
      s := _('Could not recognize file: %FILE%');
      s := ReplaceString(s,'%FILE%',file);
      ShowMessage(s);
      Exit;
   end;
   
   originalfile := file;
   pluginFile := PluginFileFromName(name, fSuffix);
   
   if FileExists(pluginFile) then
     begin
        gj2plugin_ButtonRemove.Enabled := true;
        icon := GeoJarrypluginExtractIcon(pluginFile); //load icon from the existing plugin
     end
   else 
     begin
        gj2plugin_ButtonRemove.Enabled := false;
        icon := GEOJARRY_PLUGINICON; //default icon
     end;
   
   gj2plugin_ButtonIcon.Glyph := GeoJarrypluginIconStringToBitmap(icon);
   gj2plugin_LabelName.Caption := Name;
end;

procedure GeoJarrypluginButtonIconOnClick(Sender :TObject);
var
  file : String;
begin
  if OpenDlg(_('Select icon for this plugin'), ICO_DIR , 'ofFileMustExist',_('Bitmaps 16x16px')+' (*.bmp)|*.bmp',file) then 
  begin
    icon := GeoJarrypluginBitmapToIconString(file);
    gj2plugin_ButtonIcon.Glyph := GeoJarrypluginIconStringToBitmap(icon);
  end;
end;

procedure GeoJarrypluginButtonManagerOnClick(Sender :TObject);
begin
  if pluginfile <> '' then 
  begin
    GeoJarrypluginCreate;
    GeoSetPluginGroup(pluginfile,GEOJARRY_PLUGINGROUP);
  end;
  gj2plugin.ModalResult := 1;
end;

procedure GeoJarrypluginButtonToolbarOnClick(Sender :TObject);
begin
  if pluginfile <> '' then
  begin
     GeoJarrypluginCreate;
     GeoSetPluginGroup(pluginfile,GEOJARRY_PLUGINGROUP);
     GeoSetToolbar(pluginfile,true);
  end;
  gj2plugin.ModalResult := 1;
end;

procedure GeoJarrypluginButtonRemoveOnClick(Sender :TObject);
begin
  if ((pluginfile <> '') and (MessBox(_('Do you really want to remove this plugin?'), _('Remove plugin'),4) = 6)) then
  begin
     DeleteFile(pluginfile);
     gj2plugin.ModalResult := 1;
  end;
end;

procedure GeoJarrypluginSetActions;
begin
     gj2plugin.Caption := _('Save as GeoGet plugin');
     
     gj2plugin_ButtonIcon.OnClick:= @GeoJarrypluginButtonIconOnClick;
     gj2plugin_ButtonManager.OnClick:= @GeoJarrypluginButtonManagerOnClick;
     gj2plugin_ButtonToolbar.OnClick:= @GeoJarrypluginButtonToolbarOnClick;
     gj2plugin_ButtonRemove.OnClick:= @GeoJarrypluginButtonRemoveOnClick;
end;

procedure GeoJarrypluginShow(file : String);
begin
   GeoJarrypluginSet(file);

   if (gj2plugin.showmodal = 1) then 
   begin
     ShowMessage(_('Your changes will be automatically applied immediately after you exit GeoJarry.'));
     GeoReloadPlugins;
   end;
end;
