function GeoJarrywebBookmarkGetBookmarkList : TStringList;
var
   n : Integer;
begin
   Result := TStringList.Create;
   if FileExists(GetBookmarkListFile()) then 
   begin
     Result.LoadFromFile(GetBookmarkListFile());
     for n := 0 to Result.count -1 do
     begin
         Result[n] := RegexSubstitute('\|(.+)$', Result[n],'$1');
     end;
     result.sort;
   end;
end;

function GeoJarrywebBookmarkAddBookmarkToList(name,guid : String) : boolean;
var 
  s : String;
begin
    Result := false;
    s := '';
    if ((name <> '') and (RegexFind('^' + GUID_REGEX + '$',guid))) then
    begin
       if FileExists(GetBookmarkListFile()) then s := FileToString(GetBookmarkListFile());
       if not (RegexFind(guid,s) or RegexFind('\|' + RegexQuote(name) + '(\r\n)?',s)) then
       begin
          s := s + guid + '|' + name + CRLF;
          StringToFile(s,GetBookmarkListFile());
          Result := true;
       end;
    end;
end; 

function GeoJarrywebBookmarkIsUrlValid(str : String) : boolean;
begin
   Result := RegexFind('^' + RegexQuote('http://www.geocaching.com/') + '.*bookmark.*' + GUID_REGEX + '$',str);
end;

procedure BookmarkButtonPasteOnClick(Sender : TObject);
begin
    gj2bookmark_BookmarkEditURL.Text := '';
    gj2bookmark_BookmarkEditURL.PasteFromClipboard;
    if not GeoJarrywebBookmarkIsUrlValid(gj2bookmark_BookmarkEditURL.Text) then 
    begin
        ShowMessage(_('The entered URL is not valid Geocaching.com bookmark list URL.'));
        gj2bookmark_BookmarkEditURL.Text := '';
    end;
end;

procedure BookmarkEditURLOnChange(Sender : TObject);
begin
    gj2bookmark_BookmarkButtonRecognize.enabled := GeoJarrywebBookmarkIsUrlValid(gj2bookmark_BookmarkEditURL.Text);
end;

procedure BookmarkEditNameGUIDOnChange(Sender : TObject);
begin
    gj2bookmark_BookmarkButtonAdd.Enabled := ((gj2bookmark_BookmarkEditName.Text <> '') and (RegexFind('^' + GUID_REGEX + '$',gj2bookmark_BookmarkEditGUID.Text)));
end;

procedure BookmarkButtonRecognizeOnClick(Sender : TObject);
var res,command,s : String;
begin
  //Download bookmark webpage
  //Inifile
  NewGjFilename;
  StringToFile(GetIniHeader, GetIniFile);
 
  //Command
  Command := '"' + GetGeoJarryExe + '" "' + GetIniFile + '" raw "' + gj2bookmark_BookmarkEditURL.Text +'"';
  
  //Download
  res := GeoJarryResultToString(GeoJarryRunExec(Command));
  
  If res = RESULT_VALID then
  begin
      //Retrieve the values and save them to the dialog
      s := FileToString(GetDownloadedRawFile());
      DeleteFile(GetDownloadedRawFile());
      gj2bookmark_BookmarkEditName.Text := Trim(ReplaceString(UtfToAnsi(RegexSubstitute('(?is)<\s*title\s*>([^>]*)>([^>]*)>(.*)<\s*/\s*title\s*>',s,'$3')),CRLF,''));
      gj2bookmark_BookmarkEditGUID.Text := RegexSubstitute('(' + GUID_REGEX + ')$',gj2bookmark_BookmarkEditURL.Text,'$1');
      //jump to next tab
      gj2bookmark_BookmarkPageControl.TabIndex := 1;
  end
  else ShowMessage(res);
end;

procedure BookmarkButtonAddBookmarkOnClick(Sender : TObject);
begin
    if GeoJarrywebBookmarkAddBookmarkToList(gj2bookmark_BookmarkEditName.Text,gj2bookmark_BookmarkEditGUID.Text)
    then 
    begin
      gj2web_BookmarkComboBoxBookmark.Items := GeoJarrywebBookmarkGetBookmarkList;
      gj2bookmark.close;
    end
    else ShowMessage(_('Could not add new bookmark list. The bookmark list you are trying to add might be already contained on the list.'));
end;

procedure BookmarkButtonAddOnClick(Sender : TObject);
begin
    gj2bookmark_BookmarkEditURL.Text := '';
    gj2bookmark_BookmarkEditName.Text := '';
    gj2bookmark_BookmarkEditGUID.Text := '';
    gj2bookmark_BookmarkPageControl.TabIndex := 0;
    
    gj2bookmark_BookmarkButtonRecognize.Enabled := false;
    gj2bookmark_BookmarkButtonAdd.Enabled := false;
    
    gj2bookmark.showmodal;
end;

procedure BookmarkButtonDeleteOnClick(Sender : TObject);
var
   s : String;
   list : TStringList;
   n : Integer;
begin
   s := _('Do you really want to delete the bookmark list "%NAME%"?');
   s := ReplaceString(s,'%NAME%',gj2web_BookmarkComboBoxBookmark.Text);
   if (MessBox(s,_('Delete bookmark list'),4) = 6) then
   begin
      list := TStringList.Create;
      try
        list.LoadFromFile(GetBookmarkListFile());
        for n := list.count-1 downto 0 do
        begin
            if RegexFind('^' + GUID_REGEX + '\|' + RegexQuote(gj2web_BookmarkComboBoxBookmark.Text) + '(\r\n|$)',list[n]) 
              then list.Delete(n);
        end;
        list.SaveToFile(GetBookmarkListFile());
        gj2web_BookmarkComboBoxBookmark.Items := GeoJarrywebBookmarkGetBookmarkList;
      finally
        list.Free;
      end;
   end;
end;

procedure BookmarkLabelOpenWebOnClick(Sender : TObject);
var guid : String;
begin
    guid := GetGUID(gj2web_BookmarkComboBoxBookmark.Text,GetBookmarkListFile());
    if guid <> RESULT_INVALID 
      then RunShell('http://www.geocaching.com/bookmarks/view.aspx?guid=' + guid)
      else ShowMessage(_('Error: Could not open webpage with the chosen bookmark list.'));
end;

procedure BookmarkComboBoxBookmarkOnChange(Sender : TObject);
begin
    gj2web_BookmarkComboBoxValue.Text := gj2web_BookmarkComboBoxBookmark.Text;
    gj2web_BookmarkButtonDelete.Enabled := (gj2web_BookmarkComboBoxBookmark.ItemIndex <> -1);
    gj2web_BookmarkLabelOpenWeb.Visible := (gj2web_BookmarkComboBoxBookmark.ItemIndex <> -1);
end;

procedure BookmarkCheckBoxTagOptionsEnable(enable : boolean);
begin
   gj2web_BookmarkComboBoxCategory.Enabled := enable;
   gj2web_BookmarkComboBoxValue.Enabled := enable;
   gj2web_BookmarkCheckBoxDelete.Enabled := enable;
   gj2web_BookmarkLabelCategory.Enabled := enable;
   gj2web_BookmarkLabelValue.Enabled := enable;
end;

procedure BookmarkComboBoxCategoryOnChange(Sender : TObject);
begin
   GetTagValues(gj2web_BookmarkComboBoxCategory.Text,gj2web_BookmarkComboBoxValue.Items);
end;

procedure BookmarkCheckBoxTagOnClick(Sender : TObject);
begin
    BookmarkCheckBoxTagOptionsEnable(gj2web_BookmarkCheckBoxTag.Checked);
    if gj2web_BookmarkCheckBoxTag.Checked then 
    begin
      if (gj2web_BookmarkComboBoxCategory.Text = '') then gj2web_BookmarkComboBoxCategory.Text := _('Bookmark');
      if (gj2web_BookmarkComboBoxValue.Text = '') then gj2web_BookmarkComboBoxValue.Text := gj2web_BookmarkComboBoxBookmark.Text;
    end;
end;

function GeoJarrywebBookmarkValidate : String;
begin
  Result := '';
  if not (gj2web_BookmarkCheckBoxDownload.Checked or gj2web_BookmarkCheckBoxTag.Checked) then Result := Result + _('You must set at least one of the following options') + ': "' + _('Download and import listings of all caches in the bookmark list') + '" ' + _('or') + ' "' + _('Tag caches from the bookmark list with the following values') + '". ' + _('Otherwise there would be no action to perform.')+ CRLF;
  
  if (gj2web_BookmarkComboBoxBookmark.ItemIndex = -1) then Result := Result + _('You have to select a bookmark list to download. (No "Bookmark list" selected)') + CRLF;
  if gj2web_BookmarkCheckBoxTag.Checked then
  begin
      if (gj2web_BookmarkComboBoxCategory.Text = '') then Result := Result + _('You have to set a tag category. (No "Category" selected)') + CRLF;
      //if (gj2web_BookmarkComboBoxValue.Text = '') then Result := Result + _('You have to set a tag value. (No "Value" selected)') + CRLF;
  end;

  if Result <> '' Then 
      Result := Result + RESULT_INVALID 
  else Result := RESULT_VALID;
end;

function GeoJarrywebBookmarkLoadSettings(file: String) : String;
var
  iniF: TIniFile;
begin
  Result := '';
  if FileExists(file) then
  begin
    iniF := TIniFile.Create(file);
    try
      gj2web_BookmarkComboBoxBookmark.ItemIndex := gj2web_BookmarkComboBoxBookmark.Items.IndexOf(iniF.readString('Query', 'BookmarkList', ''));
      gj2web_BookmarkCheckBoxDownload.Checked := iniF.readBool('Query', 'DownloadListings', true);
      gj2web_BookmarkCheckBoxTag.Checked := iniF.readBool('Query', 'Tag', true);
      
      if gj2web_BookmarkCheckBoxTag.Checked then
      begin
          gj2web_BookmarkComboBoxCategory.Text := iniF.readString('Query', 'TagCategory', _('Bookmark'));
          gj2web_BookmarkComboBoxValue.Text := iniF.readString('Query', 'TagValue', gj2web_BookmarkComboBoxBookmark.Text);
          gj2web_BookmarkCheckBoxDelete.Checked := iniF.readBool('Query', 'TagDelete', true);
      end;
      
      gj2web_BookmarkButtonDelete.Enabled := (gj2web_BookmarkComboBoxBookmark.ItemIndex <> -1);
      gj2web_BookmarkLabelOpenWeb.Visible := (gj2web_BookmarkComboBoxBookmark.ItemIndex <> -1);
      BookmarkComboBoxCategoryOnChange(nil);
      BookmarkCheckBoxTagOnClick(nil);
      Result := GeoJarrywebBookmarkValidate;
    finally
      iniF.Free;
    end;
  end;
end;

function GeoJarrywebBookmarkSaveSettings(file: String) : String;
var
  iniF: TIniFile;
begin
  Result := GeoJarrywebBookmarkValidate;
  if Result = RESULT_VALID then
  begin
    iniF := TIniFile.Create(file);
    try
      GeoJarrywebAllSaveQuery(file);
      iniF.writeString('Query', 'BookmarkList', gj2web_BookmarkComboBoxBookmark.Text);
      iniF.writeBool('Query', 'DownloadListings', gj2web_BookmarkCheckBoxDownload.Checked);
      iniF.writeBool('Query', 'Tag', gj2web_BookmarkCheckBoxTag.Checked);
      if gj2web_BookmarkCheckBoxTag.Checked then
      begin
          iniF.writeString('Query', 'TagCategory', gj2web_BookmarkComboBoxCategory.Text);
          iniF.writeString('Query', 'TagValue', gj2web_BookmarkComboBoxValue.Text);
          iniF.writeBool('Query', 'TagDelete', gj2web_BookmarkCheckBoxDelete.Checked);
      end;
      Result := RESULT_VALID;
    finally
      iniF.Free;
    end;
  end;
end;

{Get stringlist of geocache codes}
function GeoJarrywebBookmarkGetCacheList(var list: TStringList) : String;
var 
  command,guid : String;
  savelist : TStringList;
begin
    //Inifile
    StringToFile(GetIniHeader, GetIniFile);
    
    //GUID
    guid := GetGUID(gj2web_BookmarkComboBoxBookmark.Text,GetBookmarkListFile());
    if guid = RESULT_INVALID then  
    begin
        Result := _('Could not get GUID of the chosen bookark list: "%LIST%"');
        Result := ReplaceString(Result,'%LIST%',gj2web_BookmarkComboBoxBookmark.Text);
    end;
    
    //Command
    Command := '"' + GetGeoJarryExe + '" "' + GetIniFile + '" bookmark ' + guid + ' noloc';  
    
    //Download bookmark list
    Result := GeoJarryResultToString(GeoJarryRunExec(command));
    
    //Load list from file
    if Result = RESULT_VALID then 
    begin
      list := GetDownloadedList(GetDatadir + GetGjFilename + '.' + guid +'.txt',-1);
      //If tagging, file a request to tag after import. Do not file the request if running this method from AllButtonGgt button
      if ((gj2web_BookmarkCheckBoxTag.Checked) and (not AllButtonGgtClicked))then
      begin
          savelist := TStringList.Create;
          savelist.AddStrings(list);
          AddRequestToTagList(gj2web_BookmarkComboBoxCategory.Text,gj2web_BookmarkComboBoxValue.Text,gj2web_BookmarkCheckBoxDelete.Checked,savelist);
      end;
    end;
end;

{Download geocaches }
function GeoJarrywebBookmarkDownload : String;
var
  list : TStringList;
begin
  Result := '';
  list := TStringList.Create;
  try
     Result := GeoJarrywebBookmarkGetCacheList(list);
     if ((Result = RESULT_VALID) and (gj2web_BookmarkCheckBoxDownload.Checked))then Result := DownloadWaypointListings(list, false); //download now
  finally
     list.Free;
  end;  
end;

function GeoJarrywebBookmarkGetInfo : String;
begin
    Result := _('This query will search Geocaching.com webpage for geocaches contained in bookmark list named "%NAME%".') + ' ';
    Result := ReplaceString(Result,'%NAME%',gj2web_BookmarkComboBoxBookmark.Text);
    if gj2web_BookmarkCheckBoxDownload.Checked then Result := Result + _('Contained caches will be downloaded.') + ' ';
    if gj2web_BookmarkCheckBoxTag.Checked then 
    begin
      Result := Result + _('Caches will be tagged with category "%CATEGORY%" and value "%VALUE%".') + ' ';
      Result := ReplaceString(Result,'%CATEGORY%',gj2web_BookmarkComboBoxCategory.Text);
      Result := ReplaceString(Result,'%VALUE%',gj2web_BookmarkComboBoxValue.Text);
      if gj2web_BookmarkCheckBoxDelete.Checked then Result := Result + _('Before tagging, all existing tags of this category and value will be deleted.') + ' ';
    end;
end;

procedure GeoJarrywebBookmarkSetDefaults;
begin
     gj2web_BookmarkComboBoxBookmark.Items := GeoJarrywebBookmarkGetBookmarkList;
     gj2web_BookmarkComboBoxBookmark.ItemIndex := -1;
     
     gj2web_BookmarkButtonDelete.Enabled := false;
     gj2web_BookmarkLabelOpenWeb.Visible := false;
     
     gj2web_BookmarkComboBoxCategory.Text := _('Bookmark');
     gj2web_BookmarkComboBoxValue.Text := '';
     
     gj2web_BookmarkCheckBoxDownload.Checked := true;
     gj2web_BookmarkCheckBoxTag.Checked := true;
     gj2web_BookmarkCheckBoxDelete.Checked := true;
    
     BookmarkComboBoxCategoryOnChange(nil);
     BookmarkCheckBoxTagOnClick(nil);
end;

procedure GeoJarrywebBookmarkSetActions;
begin
   gj2web_BookmarkComboBoxCategory.OnChange := @BookmarkComboBoxCategoryOnChange;
   gj2web_BookmarkCheckBoxTag.OnClick := @BookmarkCheckBoxTagOnClick;
   gj2web_BookmarkComboBoxBookmark.OnChange := @BookmarkComboBoxBookmarkOnChange;
   GetTagCategories(gj2web_BookmarkComboBoxCategory.Items);
   gj2web_BookmarkButtonAdd.OnClick := @BookmarkButtonAddOnClick;
   gj2web_BookmarkButtonDelete.OnClick := @BookmarkButtonDeleteOnClick;
   gj2web_BookmarkLabelOpenWeb.OnClick := @BookmarkLabelOpenWebOnClick;
   
   //Add bookmark window
   gj2bookmark.Caption := _('GeoJarry %VERSION% - add bookmark list');
   gj2bookmark.Caption := ReplaceString(gj2bookmark.Caption,'%VERSION%',GEOJARRY_VERSION);
   
   gj2bookmark_BookmarkButtonPaste.OnClick := @BookmarkButtonPasteOnClick;
   gj2bookmark_BookmarkEditURL.OnChange := @BookmarkEditURLOnChange;
   gj2bookmark_BookmarkButtonRecognize.OnClick := @BookmarkButtonRecognizeOnClick;
   
   gj2bookmark_BookmarkEditName.OnChange := @BookmarkEditNameGUIDOnChange;
   gj2bookmark_BookmarkEditGUID.OnChange:= @BookmarkEditNameGUIDOnChange;
   gj2bookmark_BookmarkButtonAdd.OnClick := @BookmarkButtonAddBookmarkOnClick;
end;