var
   webdownloaded : boolean; //was the query downloaded?
   webqueryfile : string; //what file was the query loaded from?
   AllButtonGgtClicked : boolean; //is geojarry currently downloading ggt list after clicking AllButtonGgt?

{Validate the common part of the query}   
function GeoJarrywebAllValidate : String;
begin
  Result := '';
  if (gj2web_AllRGroupFound.ItemIndex = -1) then Result := Result + _('You have to select how to handle the found or the owned caches. (Unknown "Found or owned" option)') + CRLF;  
  if (gj2web_AllComboBoxType.ItemIndex = -1) then Result := Result + _('You have to select the cache type to download. (Unknown "Download geocache type" option)') + CRLF;
  
  if Result = '' Then Result := RESULT_VALID;   
end;

{Load download-related information about this query}
function GeoJarrywebAllLoadDownloadSettings(file: String) : String;
var
  iniF: TIniFile;
  res,cachetype : String;
begin
  Result := '';
  iniF := TIniFile.Create(file);
  try
    Result := isQueryCompatible(file);
    if Result = RESULT_VALID then Result := '' else Result := Result + CRLF;
    gj2web_AllRGroupFound.ItemIndex   := iniF.readInteger('Query', 'FilterOutFinds', DEFAULT_RGROUP);
    cachetype := iniF.readString('Query', 'GeocacheType', _('-- All geocache types'));
    if (cachetype = '--') then cachetype := _('-- All geocache types');
    gj2web_AllComboBoxType.ItemIndex  := gj2web_AllComboBoxType.Items.IndexOf(cachetype);
 
    res := GeoJarrywebAllValidate;
    if res = RESULT_VALID then res := '';
    
    if ((Result <> '') or (res <> '')) 
      then Result := Result + res
      else Result := RESULT_VALID;
  finally
    iniF.Free;
  end;
end;

{Save general information about this query}
procedure GeoJarrywebAllSaveQuery(file: String);
var
  iniF: TIniFile;
begin
  iniF := TIniFile.Create(file);
  try
    iniF.writeString('GeoJarry','Version', GEOJARRY_VERSION);
    iniF.writeString('Query', 'QueryBase', QUERYBASE_WEB);
    iniF.writeString('Query', 'QueryType', gj2web_AllPageControl.ActivePage.Name);
    finally
    iniF.Free;
  end;
end;

{Save download-related information about this query}
function GeoJarrywebAllSaveDownloadSettings(file: String) : String;
var
  iniF: TIniFile;
  cachetype : String;
begin
  Result := GeoJarrywebAllValidate;
  if Result = RESULT_VALID then
  begin
  iniF := TIniFile.Create(file);
    try
      iniF.writeInteger('Query', 'FilterOutFinds', gj2web_AllRGroupFound.ItemIndex);
      if RegexFind('^--',gj2web_AllComboBoxType.Text) then cachetype := '--' else cachetype := gj2web_AllComboBoxType.Text;
      iniF.writeString('Query', 'GeocacheType', cachetype);
    finally
      iniF.Free;
    end;
  end;
end;

{Translate Cache type to Groundspeak UID}
function CacheTypeToUID(ctype : String) : String;
begin
   Case ctype of
     'Traditional Cache'          : Result := '32bc9333-5e52-4957-b0f6-5a2c8fc7b257';
     'Multi-cache'                : Result := 'a5f6d0ad-d2f2-4011-8c14-940a9ebf3c74';
     'Unknown Cache'              : Result := '40861821-1835-4e11-b666-8d41064d03fe';
     'Earthcache'                 : Result := 'c66f5cf3-9523-4549-b8dd-759cd2f18db8';
     'Wherigo Cache'              : Result := '0544fa55-772d-4e5c-96a9-36a51ebcf5c9';
     'Event Cache'                : Result := '69eb8534-b718-4b35-ae3c-a856a55b0874';
     'Mega-Event Cache'           : Result := '69eb8535-b718-4b35-ae3c-a856a55b0874';
     'Cache In Trash Out Event'   : Result := '57150806-bc1a-42d6-9cf0-538d171a2d22';
     'Project APE Cache'          : Result := '2555690d-b2bc-4b55-b5ac-0cb704c0b768';
     'GPS Adventures Exhibit'     : Result := '72e69af2-7986-4990-afd9-bc16cbbb4ce3';
     'Virtual Cache'              : Result := '294d4360-ac86-4c83-84dd-8113ef678d7e';
     'Webcam Cache'               : Result := '31d2ae3c-c358-4b5f-8dcd-2185bf472d3d';
     'Letterbox Hybrid'           : Result := '4bdd8fb2-d7bc-453f-a9c5-968563b15d24';
     'Lost and Found Event Cache' : Result := '3ea6533d-bb52-42fe-b2d2-79a3424d4728';
   Else Result := RESULT_INVALID;
   End;
end;

{$INCLUDE gj2web.coord.lib.pas}
{$INCLUDE gj2web.country.lib.pas}
{$INCLUDE gj2web.user.lib.pas}
{$INCLUDE gj2web.route.lib.pas}
{$INCLUDE gj2web.name.lib.pas}
{$INCLUDE gj2web.code.lib.pas}
{$INCLUDE gj2web.pq.lib.pas}
{$INCLUDE gj2web.bookmark.lib.pas}

{Set functions for downloading, validating etc. (determine from current tab)}
procedure GeoJarrywebSetFunctions;
begin
  Case gj2web_AllPageControl.ActivePage.Name of
    'CoordTab' :
    begin
        ValidateF := @GeoJarrywebCoordValidate;
        DownloadF := @GeoJarrywebCoordDownload;
        InfoF     := @GeoJarrywebCoordGetInfo;
        SaveF     := @GeoJarrywebCoordSaveSettings;
        LoadF     := @GeoJarrywebCoordLoadSettings;
        GClistF   := @GeoJarrywebCoordGetCacheList;
    end;
    'CountryTab' : 
    begin
        ValidateF := @GeoJarrywebCountryValidate;
        DownloadF := @GeoJarrywebCountryDownload;
        InfoF     := @GeoJarrywebCountryGetInfo;
        SaveF     := @GeoJarrywebCountrySaveSettings;
        LoadF     := @GeoJarrywebCountryLoadSettings;
        GClistF   := @GeoJarrywebCountryGetCacheList;
    end;
    'UserTab' : 
    begin
        ValidateF := @GeoJarrywebUserValidate;
        DownloadF := @GeoJarrywebUserDownload;
        InfoF     := @GeoJarrywebUserGetInfo;
        SaveF     := @GeoJarrywebUserSaveSettings;
        LoadF     := @GeoJarrywebUserLoadSettings;
        GClistF   := @GeoJarrywebUserGetCacheList;
    end;
    'RouteTab' : 
    begin 
        ValidateF := @GeoJarrywebRouteValidate;
        DownloadF := @GeoJarrywebRouteDownload;
        InfoF     := @GeoJarrywebRouteGetInfo;
        SaveF     := @GeoJarrywebRouteSaveSettings;
        LoadF     := @GeoJarrywebRouteLoadSettings;
        GClistF   := @GeoJarrywebRouteGetCacheList;   
    end;
    'NameTab' : 
    begin
        ValidateF := @GeoJarrywebNameValidate;
        DownloadF := @GeoJarrywebNameDownload;
        InfoF     := @GeoJarrywebNameGetInfo;
        SaveF     := @GeoJarrywebNameSaveSettings;
        LoadF     := @GeoJarrywebNameLoadSettings;
        GClistF   := @GeoJarrywebNameGetCacheList;
    end;
    'CodeTab' : 
    begin
        ValidateF := @GeoJarrywebCodeValidate;
        DownloadF := @GeoJarrywebCodeDownload;
        InfoF     := @GeoJarrywebCodeGetInfo;
        SaveF     := @GeoJarrywebCodeSaveSettings;
        LoadF     := @GeoJarrywebCodeLoadSettings;
        GClistF   := @GeoJarrywebCodeGetCacheList;
    end;
    'PqTab' : 
    begin
        ValidateF := @GeoJarrywebPqValidate;
        DownloadF := @GeoJarrywebPqDownload;
        InfoF     := @GeoJarrywebPqGetInfo;
        SaveF     := @GeoJarrywebPqSaveSettings;
        LoadF     := @GeoJarrywebPqLoadSettings;
        GClistF   := @GeoJarrywebPqGetCacheList;
    end; 
    'BookmarkTab' : 
    begin
        ValidateF := @GeoJarrywebBookmarkValidate;
        DownloadF := @GeoJarrywebBookmarkDownload;
        InfoF     := @GeoJarrywebBookmarkGetInfo;
        SaveF     := @GeoJarrywebBookmarkSaveSettings;
        LoadF     := @GeoJarrywebBookmarkLoadSettings;
        GClistF   := @GeoJarrywebBookmarkGetCacheList;
    end;       
  end; 
end;

{Set default values into the form}
procedure GeoJarrywebSetDefaults;
begin
   gj2web_AllPageControl.TabIndex := 0; 
   
   GeoJarrywebCoordSetDefaults;
   GeoJarrywebCountrySetDefaults;
   GeoJarrywebUserSetDefaults;
   GeoJarrywebRouteSetDefaults;
   GeoJarrywebNameSetDefaults;
   GeoJarrywebCodeSetDefaults;
   GeoJarrywebPqSetDefaults;
   GeoJarrywebBookmarkSetDefaults;
   
   gj2web_AllButtonGgt.Enabled := true;
      
   gj2web_AllRGroupFound.Enabled := true;
   gj2web_AllComboBoxType.Enabled := true;
   gj2web_AllRGroupFound.ItemIndex := 0;
   gj2web_AllComboBoxType.ItemIndex := 0;
   
   webqueryfile := '';
end;

procedure GeoJarrywebAllPageControlOnChange(Sender :TObject);
var name : String;
begin
   GeoJarrywebSetFunctions;
   gj2web_AllButtonGgt.Enabled := true;
   
   name := gj2web_AllPageControl.ActivePage.Name;
   if ((name = 'CodeTab') or (name = 'PqTab') or (name = 'BookmarkTab')) then 
   begin
      if (name = 'PqTab') then gj2web_AllButtonGgt.Enabled := false;
      gj2web_AllRGroupFound.Enabled := false;
      gj2web_AllComboBoxType.Enabled := false;
      gj2web_AllRGroupFound.ItemIndex := 0;
      gj2web_AllComboBoxType.ItemIndex := 0;
   end
   else
   begin
      gj2web_AllRGroupFound.Enabled := true;
      gj2web_AllComboBoxType.Enabled := true;
   end;
end;

{Load the query into the dialog}
function GeoJarrywebLoadQuery(file : String) : String;
var
  iniF: TIniFile;
  tab : Integer;
begin
  Result := '';
  GeoJarrywebSetDefaults;
  if FileExists(file) then
  begin
    iniF := TIniFile.Create(file);
    try
        tab := TabNameToTabIndex(iniF.readString('Query', 'QueryType', ''),gj2web_AllPageControl);
        if tab <> -1 then 
        begin  
          gj2web_AllPageControl.TabIndex := tab;
          GeoJarrywebSetFunctions();
          Result := LoadF(file);
          webqueryfile := file;
          GeoJarrywebAllPageControlOnChange(nil);
        end
        else 
          begin
            Result := _('Unable to determine query type: %FILE%');
            Result := ReplaceString(Result,'%FILE%',file);
          end;
    finally
        iniF.Free;
    end;
  end 
  else 
    begin
      Result := _('No such file: %FILE%');
      Result := ReplaceString(Result,'%FILE%',file);
    end;
end;

{Show information about the query}
function GeoJarrywebQueryInfo(file : String) : String;
var s : String;
begin
  Result := GeoJarrywebLoadQuery(file);
  if Result = RESULT_VALID then
  begin
     Result := InfoF();
     if (gj2web_AllRGroupFound.ItemIndex = 1) and (gj2web_AllRGroupFound.Enabled) then Result := Result + _('Found or owned geocaches will not be downloaded.') + ' ';
     if DownloadsListings(file) then Result := Result + _('The query will download listings.') + ' ';
     if (gj2web_AllComboBoxType.ItemIndex <> 0) and (gj2web_AllComboBoxType.Enabled)then 
      begin
        s := _('Only geocaches of %TYPE% type will be downloaded.');
        s := ReplaceString(s,'%TYPE%',gj2web_AllComboBoxType.Text);
        Result := Result + s + ' ';
      end;
  end
  else 
    begin
      s := _('This query is invalid. Errors: ' + CRLF + '%ERRORS%');
      s := ReplaceString(s,'%ERRORS%',Result);
      Result := s;
    end;
end;

{Run this query}
function GeoJarrywebRunQuery: String;
begin
  NewGjFilename;
  GeoJarrywebSetFunctions;
  
  Result := ValidateF();
  if Result = RESULT_VALID then Result := DownloadF();
end;

{Run and import this query}
function GeoJarrywebRunAndImportQuery: String;
begin
  EmptyDir(GetDatadir);
  Result := GeoJarrywebRunQuery;
  if Result = RESULT_VALID then
  begin
     GeoJarryGeoImportBegin;
     GeoJarryGeoImportDir(GetDatadir);
     GeoJarryGeoImportEnd;
  end;
end;

{Load and run this query}
function GeoJarrywebLoadAndRunQuery(file : String) : String;
begin
    Result := GeoJarrywebLoadQuery(file);
    if Result = RESULT_VALID then Result := GeoJarrywebRunQuery;
end;

{Load query and get list of geocache codes}
function GeoJarrywebLoadAndGetGCList(file : String; var gclist : TStringList) : String;
begin
    Result := GeoJarrywebLoadQuery(file);
    if Result = RESULT_VALID then Result := GCListF(gclist);
end;

{Load run and import this query}
function GeoJarrywebLoadRunAndImportQuery(file : String) : String;
begin
    Result := GeoJarrywebLoadQuery(file); 
    if Result = RESULT_VALID then Result := GeoJarrywebRunAndImportQuery; 
end;

{Show me the dialog}
function GeoJarrywebShow : String;
begin
    Result := '';
    webdownloaded := false;
    gj2web.showmodal;
    if webdownloaded then Result := RESULT_DOWNLOADED;   
end;

{Load query and show in the dialog}
function GeoJarrywebLoadQueryAndShow(file : String) : String;
var s : String;
begin
    Result := GeoJarrywebLoadQuery(file);
    If Result <> RESULT_VALID then 
      begin
        s := _('This query is invalid. Errors: ' + CRLF + '%ERRORS%');
        s := ReplaceString(s,'%ERRORS%',Result);
        ShowMessage(s);
      end; 
    Result := GeoJarrywebShow;
end;

procedure GeoJarrywebAllButtonSaveOnClick(Sender :TObject);
var
  file,res : String;
begin
  GeoJarrywebSetFunctions;
  
  res := ValidateF();
  if res = RESULT_VALID then
  begin
    file := GetFile(QUERY_DIR, QUERY_SUFFIX, _('Query'), GetQueryName(webqueryfile));
    if file <> '' then SaveF(file);
  end
  else ShowMessage(res);
end;

procedure GeoJarrywebAllButtonDownloadNowOnClick(Sender :TObject);
var
  res: String;
begin
  gj2web.hide;
  res := GeoJarrywebRunAndImportQuery;
  if res <> RESULT_VALID then 
  begin
     ShowMessage(res);
     gj2web.show;
  end
  else
  begin
     webdownloaded := true;
     gj2web.close;
  end;
end;

procedure GeoJarrywebAllButtonGgtOnClick(Sender :TObject);
var
  file,res,s : String;
  list : TStringList;
begin
  //AllButtonGgtClicked
  AllButtonGgtClicked := true;
  
  GeoJarrywebSetFunctions;
  res := ValidateF();
  if res = RESULT_VALID then
  begin
    if SaveDlg(_('Select a GGT list'), GEOGET_DATADIR, 'ofFileMustExist',_('GeoGet GGT lists') + ' (*.ggt)|*.ggt',file) then 
    begin
      if not RegexFind('.ggt$', file) then file := file + '.ggt';
      s := _('%FILE% already exists. Would you like to overwrite?');
      s := ReplaceString(s,'%FILE%',file);
      if FileExists(file) then if (MessBox(s, _('List file'),4) <> 6)then  Exit;
      
      gj2web.hide;
      list := TStringList.Create;
      try
        res := GClistF(list);
        if res = RESULT_VALID then
        begin
          list.saveToFile(file);
        end else ShowMessage(res);
      finally
        list.Free;
      end;
      gj2web.show;
    end;
  end
  else ShowMessage(res);
  
  //AllButtonGgtClicked
  AllButtonGgtClicked := false;
end;

procedure GeoJarrywebSetActions;
begin
  gj2web.Caption := _('GeoJarry %VERSION% - Geocaching.com based query');
  gj2web.Caption := ReplaceString(gj2web.Caption,'%VERSION%',GEOJARRY_VERSION);

  gj2web_AllButtonSave.OnClick := @GeoJarrywebAllButtonSaveOnClick;
  gj2web_AllButtonDownloadNow.OnClick := @GeoJarrywebAllButtonDownloadNowOnClick;
  gj2web_AllPageControl.onChange := @GeoJarrywebAllPageControlOnChange;
  gj2web_AllButtonGgt.OnClick := @GeoJarrywebAllButtonGgtOnClick;
  
  GeoJarrywebCoordSetActions;
  GeoJarrywebCountrySetActions;
  GeoJarrywebUserSetActions;
  GeoJarrywebRouteSetActions;
  GeoJarrywebNameSetActions;
  GeoJarrywebCodeSetActions;
  GeoJarrywebPqSetActions;
  GeoJarrywebBookmarkSetActions;
end;
