function GeoJarrywebPqValidate : String;
begin
  Result := '';
  if (gj2web_PqComboBoxPocketQuery.Text = '') then Result := Result + _('You must select a Pocket Query to download. (No "Pocket Query to download" specified)') + CRLF;
  
  if Result <> '' Then 
      Result := Result + RESULT_INVALID 
  else Result := RESULT_VALID;
end;

procedure GeoJarrywebPqComboBoxPocketQueryOnChange(Sender : TObject);
var
  inList : boolean;
begin
  inList := ((gj2web_PqComboBoxPocketQuery.Text <> '') and (gj2web_PqComboBoxPocketQuery.Items.IndexOf(gj2web_PqComboBoxPocketQuery.Text) <> -1 ));
  gj2web_PqCheckBoxDownloadOnlyNew.Enabled := inList;
end;

function GeoJarrywebPqLoadSettings(file: String) : String;
var
  iniF: TIniFile;
begin
  Result := '';
  if FileExists(file) then
  begin
    iniF := TIniFile.Create(file);
    try
        gj2web_PqComboBoxPocketQuery.Text := iniF.readString('Query', 'PocketQuery', '');
        gj2web_PqCheckBoxDownloadOnlyNew.Checked    := iniF.readBool('Query', 'DownloadOnlyNew', false);
        
        GeoJarrywebPqComboBoxPocketQueryOnChange(nil);
        Result := GeoJarrywebPqValidate;
    finally
      iniF.Free;
    end;
  end;
end;

function GeoJarrywebPqSaveSettings(file: String) : String;
var
  iniF: TIniFile;
begin
  Result := GeoJarrywebPqValidate;
  if Result = RESULT_VALID then
  begin
    iniF := TIniFile.Create(file);
    try
      GeoJarrywebAllSaveQuery(file);
      iniF.writeString('Query', 'PocketQuery', gj2web_PqComboBoxPocketQuery.Text);
      iniF.writeBool('Query', 'DownloadOnlyNew', gj2web_PqCheckBoxDownloadOnlyNew.Checked);
      Result := RESULT_VALID;
    finally
      iniF.Free;
    end;
  end;
end;

{Get stringlist of geocache codes}
function GeoJarrywebPqGetCacheList(var list: TStringList) : String;
begin
    Result := _('List of geocache codes can not be determined from PQ GPX file.');
end;

{Get stringlist of names from list of available PQs}
function GeoJarrywebPqGetPqNamesList : TStringList;
var
   list : TStringList;
   n : Integer;
   s : String;
begin
   Result := TStringList.Create;
   Result.sorted := true;
   Result.duplicates := dupIgnore;
   if FileExists(GetPocketQueryListFile()) then
   begin
     list := TStringList.Create;
     list.LoadFromFile(GetPocketQueryListFile());
     try
        for n := 0 to list.count-1 do
         begin
             s := list[n];
             fetch(s,'|');
             Result.Add(fetch(s,'|'));
         end;
     finally
       list.Free;
     end; 
   end;
end;

{Get the most current date of creation of the given PQ}
function GeoJarrywebPqGetPqCreationDate(name : String; var date : TDateTime) : boolean;
var
  s,d : String;
  n : Integer;
begin
    Result := False;
    if FileExists(GetPocketQueryListFile()) then
    begin
      s := RegexExtract(RegexQuote(name) + '.*?' + CRLF,FileToString(GetPocketQueryListFile()));
      if s <> '' then
      begin
         for n := 1 to 4 do d := fetch(s,'|');
         Result := TryEncodeDate(StrToInt(copy(d,7,4)), StrToInt(copy(d,1,2)), StrToInt(copy(d,4,2)), date);
      end;
    end;
end;

procedure GeoJarrywebPqWritePqCreationToCache(name : String; var date : TDateTime);
var
  iniF: TIniFile;
begin
  iniF := TIniFile.Create(GetPocketQueryCreationCacheFile());
  try
    iniF.writeString('PocketQuery', name, DateToInt(date));
  finally
    iniF.Free;
  end;
end;

procedure GeoJarrywebPqReadPqCreationFromCache(name : String; var date : TDateTime);
var
  iniF: TIniFile;
begin
  iniF := TIniFile.Create(GetPocketQueryCreationCacheFile());
  try
    date := IntToDate(iniF.readString('PocketQuery', name, '20000101'));
  finally
    iniF.Free;
  end;
end;

{Download PQ of the specified name, Save list of available PQs and update GUI. If you dont set name, GJ will only download list and update GUI}
function GeoJarrywebPqGetPq(name : String) : String;
var
  Command,chosen: String;
  date: TDateTime;
begin
  Result := '';
  
  //Inifile
  StringToFile(GetIniHeader, GetIniFile);
  
  //Command
  Command := '"' + GetGeoJarryExe + '" "' + GetIniFile + '" pqdown';
  if name <> '' then Command := Command + ' "' + name + '"';
  
  //Download
  Result := GeoJarryResultToString(GeoJarryRunExec(Command));
  
  If Result = RESULT_VALID then
  begin
      //Copy
      if FileExists(GetPocketQueryListFile()) then DeleteFile(GetPocketQueryListFile());
      RenameFile(GetDownloadedPocketQueryListFile(),GetPocketQueryListFile());
      
      //Update GUI
      chosen := gj2web_PqComboBoxPocketQuery.Text;
      gj2web_PqComboBoxPocketQuery.Items := GeoJarrywebPqGetPqNamesList();
      gj2web_PqComboBoxPocketQuery.ItemIndex := gj2web_PqComboBoxPocketQuery.Items.IndexOf(chosen);
      
      //Delete downloaded HTML file to prevent GeoJarry showing useless import results dialog
      if FileExists(GetDatadir + GetGjFilename + '.pqdown.htm') then DeleteFile(GetDatadir + GetGjFilename + '.pqdown.htm');
      
      //Mark last download date - if name is in the list of PQs
      if ((name <> '') and GeoJarrywebPqGetPqCreationDate(name, date)) then GeoJarrywebPqWritePqCreationToCache(name,date);
  end;
end;

{Determines if the list of available PQs is still in the cache}
function GeojarrywebPqIsListInCache : boolean;
var
   ageSeconds : extended;
begin
    Result := false;
    If FileExists(GetPocketQueryListFile()) then
    begin
      ageSeconds := 86400 * (Now - GetFileTime(GetPocketQueryListFile()));
      Result := ageSeconds < PQ_CACHE_SECONDS;
    end;
end;

{Download Pocket Queries}
function GeoJarrywebPqDownload : String;
var
  dateLast,dateNow : TDateTime;
  download : boolean;
begin
  download := true;
  
  //We try not to download already downloaded Pocket Queries
  if gj2web_PqCheckBoxDownloadOnlyNew.Checked then
  begin
     Result := RESULT_VALID;
     
     //If list of available queries is old, renew it
     if not GeojarrywebPqIsListInCache() 
        then Result := GeoJarrywebPqGetPq('');
     
     if Result = RESULT_VALID then
     begin 
        if GeoJarrywebPqGetPqCreationDate(gj2web_PqComboBoxPocketQuery.Text,dateNow) then
        begin
          GeoJarrywebPqReadPqCreationFromCache(gj2web_PqComboBoxPocketQuery.Text,dateLast);
          //Do not download the PQ if it was not generated again since the last download
          if (dateNow <= dateLast) then download := false;
        end;
     end;
  end;
  if download then Result := GeoJarrywebPqGetPq(gj2web_PqComboBoxPocketQuery.Text);
end;


function GeoJarrywebPqGetInfo : String;
begin
    Result := _('This query will download Pocket Query named "%NAME%" from Geocaching.com webpage.');
    Result := ReplaceString(Result,'%NAME%',gj2web_PqComboBoxPocketQuery.Text) + ' ';
    if gj2web_PqCheckBoxDownloadOnlyNew.Checked then Result := Result + _('Only newly generated PQs will be downloaded.') + ' ';
end;
  
procedure GeoJarrywebPqSetDefaults;
begin
    gj2web_PqComboBoxPocketQuery.Items := GeoJarrywebPqGetPqNamesList();
    gj2web_PqComboBoxPocketQuery.ItemIndex := -1;
    gj2web_PqCheckBoxDownloadOnlyNew.Enabled := false;
    gj2web_PqCheckBoxDownloadOnlyNew.Checked := false;
end;

procedure GeoJarrywebPqLabelSetUpOnClick(Sender : TObject);
begin
    RunShell('http://www.geocaching.com/pocket');
end;

procedure GeoJarrywebPqButtonRefreshOnClick(Sender : TObject);
var
  res : String;
begin
  res := GeoJarrywebPqGetPq('');
  if (res = RESULT_VALID) then 
    begin
      res := _('List of available Pocket Queries successfully updated.' + CRLF + 'There are %COUNT% Pocket Queries available for download.');
      res := ReplaceString(res,'%COUNT%',IntToStr(gj2web_PqComboBoxPocketQuery.Items.Count));
    end;
  ShowMessage(res);  
end;

procedure GeoJarrywebPqSetActions;
begin
    gj2web_PqLabelSetUp.OnClick := @GeoJarrywebPqLabelSetUpOnClick;
    gj2web_PqButtonRefresh.OnClick := @GeoJarrywebPqButtonRefreshOnClick;
    gj2web_PqComboBoxPocketQuery.OnChange := @GeoJarrywebPqComboBoxPocketQueryOnChange;
end;