{Fetch coordinates from string and set them to x and y}
procedure GeoJarrywebCoordsFromString(var x,y: extended; line : string);
begin
    x := StrToFl(fetch(line,' '));
    y := StrToFl(fetch(line,' '));
end;

{Save given coordinates to string}
function GeoJarrywebCoordsToString(x,y: extended) : String;
begin
    Result := FlToStr(x) + ' ' + FlToStr(y);
end;

{Extract latitude from string}
function GeoJarrywebLatFromString(line : String) : String;
begin
    Result := fetch(line,' ');
end;

{Extract longitude from string}
function GeoJarrywebLonFromString(line : String) : String;
begin
    fetch(line,' ');
    Result := fetch(line,' ');
end;

{Count radius we want to use for downloading to ensure that all caches within the given distance from route will be downloaded}
function GeoJarrywebRadiusFromDistance(distance : extended) : extended;
begin
   Result := distance * 1.15;
end;

{Extract current radius from form and convert it to kilometers}
function GeoJarrywebRadiusInKilometers : extended;
begin
  Result := StrToFloat(gj2web_RouteComboBoxDistance.Text);
  case gj2web_RouteComboBoxUnit.Text of
    'km': begin ; end;
    'mi': begin Result := Result*MILE; end;
  else ShowMessage(_('Could not determine distance unit. Using kilometers!'));
  end;
  Result := GeoJarrywebRadiusFromDistance(Result);
end;

{Extract current radius from form and convert it to miles}
function GeoJarrywebRadiusInMiles : extended;
begin
  Result := StrToFloat(gj2web_RouteComboBoxDistance.Text);
  case gj2web_RouteComboBoxUnit.Text of
    'km': begin Result := (Result/MILE); end;
    'mi': begin ; end;
  else ShowMessage(_('Could not determine distance unit. Using miles!'));
  end;
  Result := GeoJarrywebRadiusFromDistance(Result);
end;

procedure GeoJarrywebRouteComboBoxRouteOnChange(Sender : TObject);
var
    enabled : boolean;
begin
    enabled := gj2web_RouteComboBoxRoute.ItemIndex <> -1;
    gj2web_RouteButtonPreviewSource.Enabled := enabled;
    gj2web_RouteButtonPreviewOptimized.Enabled := ((enabled) and (gj2web_RouteComboBoxDistance.ItemIndex <> -1));
end;

function GeoJarrywebGetRadiusCaption : String;
begin
    Result := '';
    case gj2web_RouteComboBoxUnit.Text of
      '' : begin Result := _('Set distance first'); end;
      'km' : begin Result := fltostr(GeoJarrywebRadiusInKilometers) + ' km'; end;
      'mi' : begin Result := fltostr(GeoJarrywebRadiusInMiles) + ' mi'; end;
      else ShowMessage(_('Could not determine distance unit. Using miles!'));
    end;
end;

procedure GeoJarrywebRouteComboBoxDistanceOnChange(Sender : TObject);
begin
    gj2web_RouteButtonPreviewOptimized.Enabled := ((gj2web_RouteComboBoxRoute.ItemIndex <> -1) and (gj2web_RouteComboBoxDistance.ItemIndex <> -1));
    if (gj2web_RouteComboBoxDistance.ItemIndex <> -1) then gj2web_RouteLabelRadius.Caption := GeoJarrywebGetRadiusCaption;
end;


{Load saved route of given name into stringlist}
function GeoJarrywebGetSavedRoute(name : String) : TStringList;
begin
   Result := TStringList.Create;
   Result.LoadFromFile(GEOGET_DATADIR + ROUTE_DIR + name + '.txt');
end;

{Format given route as GPX file}
function GeoJarrywebRouteToGPX(route : TStringList) : String;
var
  c,w,l : Integer;
begin
  w := 0;
  l := Length(IntToStr(route.count)); //we will prefix the name of waypoints with adequate number of leading zeros
  Result := 
      '<?xml version="1.0" encoding="utf-8"?>' + CRLF 
    + '<gpx xmlns:xsd="http://www.w3.org/2001/XMLSchema"'
    + ' xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" version="1.0"'
    + ' creator="' + GEOGET_VERSION + '"'
    + ' xsi:schemaLocation="http://www.topografix.com/GPX/1/0 http://www.topografix.com/GPX/1/0/gpx.xsd"'
    + ' xmlns="http://www.topografix.com/GPX/1/0">' + CRLF;
  Result := Result + ' <desc>Geocache file</desc>' + CRLF;
  for c := 0 to route.count-1 do
  begin
     if RegexFind('^\s*#', route[c]) then continue; //comment 
     w := w + 1; //number of exported waypoints
     Result := Result + 
     '  <wpt lat="'+ GeoJarrywebLatFromString(route[c])+ '" lon="'+ GeoJarrywebLonFromString(route[c]) + '">' + CRLF +
     '    <name><![CDATA[Route waypoint ' + PrefixToLength('0',l,IntToStr(w)) + ']]></name>' + CRLF +
     '  </wpt>' + CRLF + CRLF;
  end;
  Result := Result + '</gpx>';
end;

{Optimize the given road to the selected interval of circle centres}
function GeoJarrywebGetCentralPoints(route: TStringList; interval : extended) : TStringList;
var
   x,y,xnext,ynext,xnew,ynew,azimuth : extended;
   n : integer;  
begin
   Result := TStringList.Create;
   for n := 0 to route.count-1 do
   begin
      if RegexFind('^\s*#', route[n]) then continue; //comment
   
      //load waypoint from the route
      GeoJarrywebCoordsFromString(x,y,route[n]);
      Result.add(GeoJarrywebCoordsToString(x,y)); //this waypoint will be included in the result
      if (n = route.count-1) then continue; //the route ends here
 
      //load next waypoint from the route
      GeoJarrywebCoordsFromString(xnext,ynext,route[n+1]);
 
      while PointDistanceEx(x, y, xnext, ynext, azimuth) > interval do
      begin
          PointProject(x,y,azimuth,interval,xnew,ynew); //project waypoint in the given azimuth and distance 
          x := xnew; y := ynew; //the projected waypoint is our new starting point
          Result.add(GeoJarrywebCoordsToString(x,y)); //projected waypoint will be included in the result
      end;
   end;
   
   for n := result.count-2 downto 1 do //we don't want to filter the first or the last waypoint in the route
   begin
     GeoJarrywebCoordsFromString(x,y,result[n+1]);
     GeoJarrywebCoordsFromString(xnext,ynext,result[n-1]);
     if PointDistance(x,y,xnext,ynext) < interval then result.delete(n); //filter out waypoints too close to each other
   end; 
end;

function GeoJarrywebRouteValidate : String;
var
  gjwebResult : String;
begin
  Result := '';
  if (gj2web_RouteComboBoxRoute.Text = '') then Result := Result + _('You have to select a route. (No "Route" selected or file not found)') + CRLF;
  if (gj2web_RouteComboBoxDistance.Text = '') then Result := Result + _('You have to select the maximum distance. (No "Distance" selected)') + CRLF;
  if (gj2web_RouteComboBoxUnit.Text = '') then Result := Result + _('You have to select the units. (No "Units" selected)') + CRLF;
  
  gjwebResult := GeoJarrywebAllValidate;
  if gjwebResult <> RESULT_VALID then Result := Result + gjwebResult;

  if Result <> '' Then 
      Result := Result + RESULT_INVALID 
  else Result := RESULT_VALID;  
end;

function GeoJarrywebRouteLoadSettings(file: String) : String;
var
  iniF: TIniFile;
begin
  Result := GeoJarrywebAllLoadDownloadSettings(file)
  if FileExists(file) and (Result = RESULT_VALID) then
  begin
    iniF := TIniFile.Create(file);
    try
      gj2web_RouteComboBoxRoute.ItemIndex        := gj2web_RouteComboBoxRoute.Items.indexOf(iniF.readString('Query', 'Route', ''));
      gj2web_RouteComboBoxDistance.ItemIndex     := gj2web_RouteComboBoxDistance.Items.indexOf(iniF.readString('Query', 'Distance', ''));
      gj2web_RouteComboBoxUnit.ItemIndex         := gj2web_RouteComboBoxUnit.Items.indexOf(iniF.readString('Query', 'DistanceUnit', gDistanceUnit));
      GeoJarrywebRouteComboBoxRouteOnChange(nil);
      GeoJarrywebRouteComboBoxDistanceOnChange(nil);
      
      Result := GeoJarrywebRouteValidate;
    finally
      iniF.Free;
    end;
  end;
end;

function GeoJarrywebRouteSaveSettings(file: String) : String;
var
  iniF: TIniFile;
begin
  Result := GeoJarrywebRouteValidate;
  if Result = RESULT_VALID then
  begin
    iniF := TIniFile.Create(file);
    try
       GeoJarrywebAllSaveQuery(file);
       iniF.writeString('Query', 'Route', gj2web_RouteComboBoxRoute.Text);
       iniF.writeString('Query', 'Distance', gj2web_RouteComboBoxDistance.Text);
       iniF.writeString('Query', 'DistanceUnit', gj2web_RouteComboBoxUnit.Text);
       Result := GeoJarrywebAllSaveDownloadSettings(file);
    finally
      iniF.Free;
    end;
  end;
end;


procedure GeoJarrywebRoutePrepareIniFile;
var
  IniBody, IniFile : String;
begin
  IniFile := GetIniFile;
  IniBody := GetIniHeader;
  StringToFile(IniBody, IniFile);
end;

function GeoJarrywebRoutePrepareCommand(lat,lon : string) : String;
var
  ctype : String;
begin
  Result := '"' + GetGeoJarryExe + '" "' + GetIniFile + '"';
  Result := Result + ' gclist "lat=' + lat + '&lon=' + lon;  
  if (gj2web_AllRGroupFound.ItemIndex = 1) then Result := Result + '&f=1';
  ctype := CacheTypeToUID(gj2web_AllComboBoxType.Text);
  if (ctype <> RESULT_INVALID) then Result := Result + '&tx=' + ctype;
  Result := Result + '&dist=' + fltostr(GeoJarrywebRadiusInMiles);
  Result := Result + '"';
end;

{Get stringlist of geocache codes}
function GeoJarrywebRouteGetCacheList(var list: TStringList) : String;
var
  n,max : Integer;
  centerlist : TStringList;
  command,lat,lon : String;
begin
    list.Clear;  //just to be sure
    list.sorted := true; //has to be true to enable dupIgnore feature
    list.duplicates := dupIgnore //we don't want to download one geocache listing more than once
    
    GeoBusyCaption(_('GeoJarry'));
    GeoBusyKind(_('Optimizing route waypoints'));
    
    centerlist := GeoJarrywebGetCentralPoints(GeoJarrywebGetSavedRoute(gj2web_RouteComboBoxRoute.Text), GeoJarrywebRadiusInKilometers);
    try
      GeoJarrywebRoutePrepareIniFile;
      max := centerlist.count - 1;
      for n := 0 to max do
      begin
        GeoBusyProgress(n+1,max+1);
        if RegexFind('^\s*#', centerlist[n]) then continue; //comment
        
        lat := GeoJarrywebLatFromString(centerlist[n]);
        lon := GeoJarrywebLonFromString(centerlist[n]);
        GeoBusyKind(_('Current position') + ': ' + FormatCoordNum(StrToFloat(lat),StrToFloat(lon)));
        
        command := GeoJarrywebRoutePrepareCommand(lat, lon);
        
        Result := GeoJarryResultToString(GeoJarryRunExec(command));
        if Result = RESULT_VALID  
          then list.AddStrings(GetDownloadedList(GetDownloadedGcListFile,-1))
          else if Result = RESULT_NOWAYPOINTS then Result := RESULT_VALID
          else Exit;
      end;
   finally
      GeoBusyCaption('');
      GeoBusyKind('');
      GeoBusyValue('');
      centerlist.Free;
   end;
end;

{Download geocaches }
function GeoJarrywebRouteDownload : String;
var
  list : TStringList;
begin
  Result := '';
  list := TStringList.Create;
  try
     Result := GeoJarrywebRouteGetCacheList(list);
     if Result = RESULT_VALID then Result := DownloadWaypointListings(list, false); //download now
  finally
     list.Free;
  end;
end;

procedure GeoJarrywebRouteButtonPreviewSourceOnClick(Sender : TObject);
var
  file : String;
begin
  if SaveDlg(_('Select a GPX file to save the route'), '%HOMEPATH%', '',_('GPX files') + ' (*.gpx)|*.gpx',file) then
  begin
    if not RegexFind('.gpx$', file) then file := file + '.gpx';
    StringToFile(GeoJarrywebRouteToGPX(GeoJarrywebGetSavedRoute(gj2web_RouteComboBoxRoute.Text)), file);
  end;
end;

procedure GeoJarrywebRouteButtonPreviewOptimizedOnClick(Sender : TObject);
var
  file : String;
begin
  if SaveDlg(_('Select a GPX file to save the route'), '%HOMEPATH%', '',_('GPX files') +' (*.gpx)|*.gpx',file) then
  begin
    if not RegexFind('.gpx$', file) then file := file + '.gpx';
    StringToFile(GeoJarrywebRouteToGPX(GeoJarrywebGetCentralPoints(GeoJarrywebGetSavedRoute(gj2web_RouteComboBoxRoute.Text), GeoJarrywebRadiusInKilometers)), file);
  end;
end;

function GeoJarrywebRouteGetInfo : String;
begin
    Result := _('This query will search Geocaching.com webpage for geocaches along route "%ROUTE%" up to distance of %DISTANCE%%UNIT%.') + ' ';
    Result := ReplaceString(Result,'%ROUTE%',gj2web_RouteComboBoxRoute.Text);
    Result := ReplaceString(Result,'%DISTANCE%',gj2web_RouteComboBoxDistance.Text);
    Result := ReplaceString(Result,'%UNIT%',gj2web_RouteComboBoxUnit.Text);
end;

procedure GeoJarrywebRouteSetDefaults;
begin
     gj2web_RouteComboBoxRoute.Items := GetFilteredFileList(GEOGET_DATADIR + ROUTE_DIR, 'txt');
     gj2web_RouteComboBoxUnit.ItemIndex := gj2web_RouteComboBoxUnit.Items.indexOf(gDistanceUnit);
     gj2web_RouteLabelRadius.Caption := _('Set distance first');
     
     gj2web_RouteComboBoxDistance.ItemIndex := -1;
     gj2web_RouteComboBoxRoute.ItemIndex := -1;
     gj2web_RouteButtonPreviewSource.Enabled := false;
     gj2web_RouteButtonPreviewOptimized.Enabled := false;
end;

procedure GeoJarrywebRouteSetActions;
begin
    GeoJarrywebRouteSetDefaults;
    gj2web_RouteComboBoxRoute.OnChange := @GeoJarrywebRouteComboBoxRouteOnChange;
    gj2web_RouteComboBoxDistance.OnChange := @GeoJarrywebRouteComboBoxDistanceOnChange;
    gj2web_RouteComboBoxUnit.OnChange := @GeoJarrywebRouteComboBoxDistanceOnChange;
    gj2web_RouteButtonPreviewSource.OnClick := @GeoJarrywebRouteButtonPreviewSourceOnClick;
    gj2web_RouteButtonPreviewOptimized.OnClick := @GeoJarrywebRouteButtonPreviewOptimizedOnClick;
end; 