function GeoJarrywmCategoryValidate : String;
begin
    Result := '';
    if (gj2wm_CategoryComboBox.Text = '') then Result := Result + _('You have to select a category. (No "Category" found)') + CRLF;
  
    if Result = '' then Result := RESULT_VALID;
end;

function GeoJarrywmCategoryLoad(file: String) : String;
var
  iniF: TIniFile;
begin
  Result := '';
  if FileExists(file) then
  begin
    iniF := TIniFile.Create(file);
    try
      gj2wm_CategoryComboBox.ItemIndex   := gj2wm_CategoryComboBox.Items.IndexOf(iniF.readString('Query', 'Category', ''));
      
      Result := GeoJarrywmCategoryValidate;
    finally
      iniF.Free;
    end;
  end;
end;

function GeoJarrywmCategorySave(file: String) : String;
var
  iniF: TIniFile;
begin
  Result := GeoJarrywmCategoryValidate;
  if Result = RESULT_VALID then
  begin
    iniF := TIniFile.Create(file);
    try
      iniF.writeBool('Criteria','Category', true);
      iniF.writeString('Query', 'Category', gj2wm_CategoryComboBox.Text);
    finally
      iniF.Free;
    end;
  end;
end;

function GeoJarrywmCategoryGetInfo : String;
begin
    Result := _('Category: %CATEGORY%');
    ReplaceString(Result,'%CATEGORY%',gj2wm_CategoryComboBox.Text);
end;

function GeoJarrywmCategoryGetUrlParameters(var list: TStringList) : String;
begin
    Result := GetGUID(gj2wm_CategoryComboBox.Text,GetWaymarkCategoryListFile());
    if Result <> RESULT_INVALID then
    begin
       list.add('guid=' + EncodeUrlElement(Result));
       Result := RESULT_VALID;
    end 
    else 
      begin
        Result := _('Could not find ID of category "%CATEGORY%".');
        Result := ReplaceString(Result,'%CATEGORY%',gj2wm_CategoryComboBox.Text);
      end; 
end;

function GeoJarrywmCategoryGetCategoryList : TStringList;
var
   n : Integer;
begin
   Result := TStringList.Create;
   if FileExists(GetWaymarkCategoryListFile()) then 
   begin
     Result.LoadFromFile(GetWaymarkCategoryListFile());
     for n := 0 to Result.count -1 do
     begin
         Result[n] := RegexSubstitute('\|(.+)$', Result[n],'$1');
     end;
     Result.Sort;
   end
   else ShowMessage(_('Could not find file containing list of waymark categories. Refresh the list manually from the web.'));
end;

procedure GeoJarrywmCategorySetDefaults;
begin
     gj2wm_CategoryComboBox.ItemIndex := -1;
     gj2wm_CategoryComboBox.Text := '';
end;

function GeoJarrywmCategoryRefreshListFromWeb : String;
var
  Command: String;
  formerCount : Longint;
begin
  Result := '';
  
  //Inifile
  StringToFile(GetIniHeader, GetIniFile);
  
  //Command
  Command := '"' + GetGeoJarryExe + '" "' + GetIniFile + '" wmcategory';
  
  //Download
  Result := GeoJarryResultToString(GeoJarryRunExec(Command));
  
  If Result = RESULT_VALID then
  begin
      if FileExists(GetWaymarkCategoryListFile()) then 
      begin
        formerCount := GetNumberOfLines(GetWaymarkCategoryListFile());
        DeleteFile(GetWaymarkCategoryListFile());
      end;
      RenameFile(GetDownloadedWaymarkCategoryListFile(),GetWaymarkCategoryListFile());
      gj2wm_CategoryComboBox.Items := GeoJarrywmCategoryGetCategoryList();
      gj2wm_CategoryComboBox.ItemIndex := -1;
      Result := _('List of waymark categories has been successfully updated.' + CRLF + 'Categories added: %ADDED%' + CRLF + 'Categories total: %TOTAL%');
      Result := ReplaceString(Result,'%ADDED%',IntToStr(gj2wm_CategoryComboBox.Items.Count - formerCount));
      Result := ReplaceString(Result,'%TOTAL%',IntToStr(gj2wm_CategoryComboBox.Items.Count));
  end;
end;

procedure GeoJarrywmCategoryButtonRefreshOnClick(Sender : TObject);
begin
   ShowMessage(GeoJarrywmCategoryRefreshListFromWeb);
end;

procedure GeoJarrywmCategorySetActions;
begin
     gj2wm_CategoryButtonRefresh.OnClick := @GeoJarrywmCategoryButtonRefreshOnClick;
     gj2wm_CategoryComboBox.Items := GeoJarrywmCategoryGetCategoryList;
end;
